@@ -4,6 +4,7 @@ from ase.io import read,write
 from .molecule import molecules
 from ase.io.trajectory import TrajectoryWriter,Trajectory
 from ase.calculators.singlepoint import SinglePointCalculator
+#from ase.visualize import view
 import numpy as np
 import matplotlib.pyplot as plt
 
@@ -18,7 +19,6 @@ def getAtomsToMove(i,j,j_,ToBeMove,neighbors,ring=False):
            ring = True
     return ToBeMove,ring
 
-
 def get_group(i,j,atoms):
     positions = atoms.get_positions()
     center    = 0.5*(positions[i] + positions[j]) 
@@ -38,7 +38,6 @@ def get_group(i,j,atoms):
            group_j.append(i_)
     return group_i,group_j
 
-
 def getNeighbor(natom,r,rcut,bo,botol=0.0):
     neighbors = [[] for _ in range(natom)]
     for i in range(natom-1):
@@ -50,7 +49,6 @@ def getNeighbor(natom,r,rcut,bo,botol=0.0):
                neighbors[j].append(i)
     return neighbors
 
-
 def getBonds(natom,r,rcut,bo,botol=0.0):
     bonds = [] 
     for i in range(natom-1):
@@ -60,7 +58,6 @@ def getBonds(natom,r,rcut,bo,botol=0.0):
                bonds.append((i,j))
     return bonds
 
-
 def add_zmat_atom(fi,se,th,neighbors,zmat_id,zmat_index):
     for i in neighbors[fi]:
         if i not in zmat_id:
@@ -69,7 +66,6 @@ def add_zmat_atom(fi,se,th,neighbors,zmat_id,zmat_index):
            zmat_id,zmat_index=add_zmat_atom(i,fi,se,neighbors,zmat_id,zmat_index)
     return zmat_id,zmat_index
 
-
 def get_zmat_variable(i,j,k,l,positions):
     vij  = positions[i]-positions[j]
     rij  = np.sqrt(np.sum(np.square(vij)))
@@ -89,24 +85,29 @@ def get_zmat_variable(i,j,k,l,positions):
 
     vi   = np.cross(uij,ukj)
     ri   = np.sqrt(np.sum(np.square(vi)))
-    ui   = vi/ri
-
-    vj   = np.cross(ulk,ujk)
-    rj   = np.sqrt(np.sum(np.square(vj)))
-    uj   = vj/rj
-    
-    cos_ = np.dot(ui,uj)
-    vk   = np.cross(ui,uj)
-    rk   = np.sqrt(np.sum(np.square(vk)))
-    if rk==0.0:
+    if ri <=0.00000001:
        tor = 0.0
     else:
-       uk   = vk/rk
-       # print(ui,uj,cos_,vk,uk)
-       s    = np.dot(uk,ujk)
-       tor  = s*np.arccos(cos_)*180.0/3.141593
-    return rij,ang,tor
+       ui   = vi/ri
 
+       vj   = np.cross(ulk,ujk)
+       rj   = np.sqrt(np.sum(np.square(vj)))
+       if rj<=0.00000001:
+          tor = 0.0
+       else:
+          uj   = vj/rj
+         
+          cos_ = np.dot(ui,uj)
+          vk   = np.cross(ui,uj)
+          rk   = np.sqrt(np.sum(np.square(vk)))
+          if rk==0.0:
+             tor = 0.0
+          else:
+             uk   = vk/rk
+             # print(ui,uj,cos_,vk,uk)
+             s    = np.dot(uk,ujk)
+             tor  = s*np.arccos(cos_)*180.0/3.141593
+    return rij,ang,tor
 
 def get_zmat_angle(i,j,k,positions):
     vij  = positions[i]-positions[j]
@@ -125,7 +126,6 @@ def get_zmat_angle(i,j,k,positions):
     ang  = np.arccos(cos_)*180.0/3.141593
     return rij,ang
 
-
 def get_zmatrix(atoms,zmat_id,zmat_index):
     ''' get the zmatrix of a molecular'''
     zmatrix = []
@@ -148,9 +148,9 @@ def get_zmatrix(atoms,zmat_id,zmat_index):
            zmatrix.append([r,ang,tor])
     return zmatrix
 
- 
-def check_zmat(atoms=None,rmin=0.8,rmax=1.2,angmax=15.0,
+def check_zmat(atoms=None,rmin=0.8,rmax=1.35,angmax=25.0,
                zmat_id=None,zmat_index=None,InitZmat=None):
+    '''  Check for severe deformation '''
     zmatrix = get_zmatrix(atoms,zmat_id,zmat_index)
     # print(zmatrix)
     Df_     = 0
@@ -162,63 +162,63 @@ def check_zmat(atoms=None,rmin=0.8,rmax=1.2,angmax=15.0,
     zv      = None
     for i in range(len(zmat_id)):
         zvlo    = 0.0
-        if zmat_index[i][2]!=-1:                    # check torsion
-           ang = InitZmat[i][2]-zmatrix[i][2]
-           if ang>180.0:
-              ang = ang - 360.0
-           elif ang<-180.0:
-              ang = ang + 360.0
-           ang_ = abs(ang)
-           if ang_>=1.5*angmax: 
-              Df_ += 0.6
-              zv   = (i,2)
-              zvhi = ang
-              zvs.append(zv)
-              score.append(0.6)
-              zvlo_.append(zvlo)
-              zvhi_.append(zvhi)
-           elif ang_>=angmax: 
-              Df_ += 0.3
+      #   if zmat_index[i][2]!=-1:                    # check torsion
+      #      ang = InitZmat[i][2]-zmatrix[i][2]
+      #      if ang>180.0:
+      #         ang = ang - 360.0
+      #      elif ang<-180.0:
+      #         ang = ang + 360.0
+      #      ang_ = abs(ang)
+      #      if ang_>=1.5*angmax: 
+      #         Df_ += 0.6
+      #         zv   = (i,2)
+      #         zvhi = ang
+      #         zvs.append(zv)
+      #         score.append(0.6)
+      #         zvlo_.append(zvlo)
+      #         zvhi_.append(zvhi)
+      #      elif ang_>=angmax: 
+      #         Df_ += 0.3
 
         if zmat_index[i][1]!=-1:                    # check angle
            ang  = InitZmat[i][1]-zmatrix[i][1]
            ang_ = abs(ang)
-           if ang_>=1.5*angmax: 
-              Df_ += 0.8
+           if ang_>=angmax: 
+              Df_ += 1.0
               zv   = (i,1)
               zvhi = ang
               zvs.append(zv)
               score.append(0.8)
               zvlo_.append(zvlo)
               zvhi_.append(zvhi)
-           elif ang_>=angmax: 
-              Df_ += 0.5
-              zv   = (i,1)
-              zvhi = ang
-              zvs.append(zv)
-              score.append(0.5)
-              zvlo_.append(zvlo)
-              zvhi_.append(zvhi)
+         #   elif ang_>=angmax: 
+         #      Df_ += 0.5
+         #      zv   = (i,1)
+         #      zvhi = ang
+         #      zvs.append(zv)
+         #      score.append(0.5)
+         #      zvlo_.append(zvlo)
+         #      zvhi_.append(zvhi)
 
         if zmat_index[i][0]!=-1: 
            r_   = zmatrix[i][0]/InitZmat[i][0]
            zvhi = InitZmat[i][0] - zmatrix[i][0]
-           if r_<=0.9*rmin or r_>=1.1*rmax:
-              Df_ += 1.0
+           if r_<=rmin or r_>=rmax:
+              Df_ += 1.1
               zv   = (i,0)
               zvs.append(zv)
               score.append(1.0)
               zvlo_.append(zvlo)
               zvhi_.append(zvhi)
-           elif r_<=rmin or r_>=rmax:
-              Df_ += 0.64
-              zv   = (i,0)
-              zvs.append(zv)
-              score.append(0.64)
-              zvlo_.append(zvlo)
-              zvhi_.append(zvhi)
-           elif r_<=1.1*rmin or r_>=0.9*rmax: 
-              Df_ += 0.35
+         #   elif r_<=rmin or r_>=rmax:
+         #      Df_ += 0.64
+         #      zv   = (i,0)
+         #      zvs.append(zv)
+         #      score.append(0.64)
+         #      zvlo_.append(zvlo)
+         #      zvhi_.append(zvhi)
+         #   elif r_<=1.1*rmin or r_>=0.9*rmax: 
+         #      Df_ += 0.35
     if len(score)>=1:
        m = np.argmax(score)
        zv,zvlo,zvhi = zvs[m],zvlo_[m],zvhi_[m] 
@@ -277,6 +277,7 @@ class AtomDance(object):
                                    botol=self.botol)
       
       self.InitZmat = np.array(self.get_zmatrix(atoms))
+      self.zmatrix  = None
       # self.write_zmat(self.InitZmat)
 
   def get_zmatrix(self,atoms):
@@ -364,8 +365,10 @@ class AtomDance(object):
              continue
           elif self.zmat_index[i][0]!=-1 and self.zmat_index[i][1]==-1 and self.zmat_index[i][2]==-1:
              atoms = self.stretch_atom(atomi,atomj,zmat[i][0],atoms)
+             # view(atoms)
           elif self.zmat_index[i][0]!=-1 and self.zmat_index[i][1]!=-1 and self.zmat_index[i][2]==-1:
              atoms = self.rotate_atom(atoms,atomi,atomj,atomk,atoml,r=r,ang=ang)
+             # view(atoms)
           else:
              atoms = self.rotate_atom(atoms,atomi,atomj,atomk,atoml,r=r,ang=ang,tor=tor)
       return atoms
@@ -382,35 +385,46 @@ class AtomDance(object):
       vkj = atoms.positions[k] - atoms.positions[j]
       rkj = np.sqrt(np.sum(np.square(vkj)))
       rij = np.sqrt(np.sum(np.square(vij)))
-      
+
       ux  = vkj/rkj
+      # print(ux)
       if tor is None or tor == 0.0:
          uij = vij/rij
          rk  = np.dot(uij,ux)
+         #print(rk)
          vy  = uij - rk*ux
-         uy  = vy/np.sqrt(np.sum(np.square(vy)))
+         ry  = np.sqrt(np.sum(np.square(vy)))
+         if ry>0.00000001:  # paralell
+            uy = vy/ry
+         #print(uy)
       else:
          vkl = atoms.positions[k] - atoms.positions[l] 
          rkl = np.sqrt(np.sum(np.square(vkl)))
          ukl = vkl/rkl
          rk  = np.dot(ukl,ux)
          vy  = ukl - rk*ux
-         uy  = vy/np.sqrt(np.sum(np.square(vy))) 
+         ry  = np.sqrt(np.sum(np.square(vy)))
+         if ry>0.00000001:
+            uy = np.array([0.0,0.0,1.0])
+         else:
+            uy  = vy/ry 
 
-      a   = ang*3.141593/180.0
-      ox  = r*np.cos(a)*ux
-      ro  = r*np.sin(a)
-      oy  = ro*uy
-      p   = ox + oy
-      atoms.positions[i] = atoms.positions[j] + p
+      if ry>0.00000001 and abs(ang - 180.0)>0.00000001:
+         a   = ang*3.141593/180.0
+         ox  = r*np.cos(a)*ux
+         ro  = r*np.sin(a)
+         oy  = ro*uy
+         p   = ox + oy
+         atoms.positions[i] = atoms.positions[j] + p
 
-      if not tor is None:
-         vij = p
-         uz  = np.cross(ux,uy)
-         o_  = atoms.positions[j] + ox
-         a   = tor*3.141593/180.0
-         p   = ro*np.cos(a)*uy + ro*np.sin(a)*uz
-         atoms.positions[i] = o_ + p
+      if not tor is None :
+         if abs(tor) >0.00000001 and ry>0.00000001:
+            vij = p
+            uz  = np.cross(ux,uy)
+            o_  = atoms.positions[j] + ox
+            a   = tor*3.141593/180.0
+            p   = ro*np.cos(a)*uy + ro*np.sin(a)*uz
+            atoms.positions[i] = o_ + p
       return atoms
 
   def get_rotate(self):
@@ -581,25 +595,30 @@ class AtomDance(object):
       atoms.set_velocities(v)
       return atoms,group_i,group_j
 
-  def zmat_relax(self,atoms,nbin=10,relax_step=None,
+  def zmat_relax(self,atoms,zmatrix=None,nbin=10,relax_step=None,
                  zmat_variable=None,zvlo=None,zvhi=None,
                  traj='zmat.traj',relaxlog = '',reset=False):
       atoms_ = atoms.copy()
-      zmatrix  = np.array(self.get_zmatrix(atoms))
+      if zmatrix is None:
+         zmatrix  = np.array(self.get_zmatrix(atoms))
+    
       initz    = np.array(self.InitZmat)         # target
       relax_step_ = nbin if relax_step is None else relax_step
       if not zmat_variable is None:
          i,j           = zmat_variable
-         initz         = zmatrix.copy()
-         initz[i][j]   = initz[i][j] + zvhi     # target
-         zmatrix[i][j] = zmatrix[i][j] + zvlo   # starting point: current configration
-         relaxlog += '                     ---------------------------\n'
-         relaxlog += '                         %d-%d-%d-%d (%d,%d) \n' %(self.zmat_id[i],self.zmat_index[i][0],
+         initz         = zmatrix.copy() 
+         r             = initz[i][j]
+         relaxlog += '      --------------------------------------------------------\n'
+         relaxlog += '\n                         %d-%d-%d-%d (%d,%d) \n' %(self.zmat_id[i],self.zmat_index[i][0],
                                                                    self.zmat_index[i][1],self.zmat_index[i][2],i,j)
-         relaxlog += '                     ---------------------------\n'
-         relaxlog += 'varied from: %8.4f to %8.4f,\n' %(zmatrix[i][j],initz[i][j])
+         relaxlog += '\n               the current Zvariable value: {:8.4f}   \n'.format(r)
+         initz[i][j]   = r + zvhi   # target
+         zmatrix[i][j] = r + zvlo   # starting point: current configration
+         relaxlog += '\n      --------------------------------------------------------\n'
+         relaxlog += '\n       zmat vairiable will changed by {:8.4f}  to  {:8.4f} \n'.format(zvlo,zvhi)
+         relaxlog += '                  varied from: {:8.4f} to {:8.4f} \n'.format(zmatrix[i][j],initz[i][j])
       else:
-         relaxlog += 'relax the structure to %d/%d of the initial value ...\n' %(relax_step_,nbin)
+         relaxlog += '      relax the structure to {:2d}/{:2d} of the initial value ...\n'.format(relax_step_,nbin)
       dz_  = (initz - zmatrix)
       dz_     = np.where(dz_>180.0,dz_-360.0,dz_)
       dz      = np.where(dz_<-180.0,dz_+360.0,dz_)
@@ -617,8 +636,8 @@ class AtomDance(object):
 
           atoms = self.zmat_to_cartation(atoms,zmat_)
           self.ir.calculate(atoms)
-          calc = SinglePointCalculator(atoms,energy=self.ir.E)
-          atoms.set_calculator(calc)
+          atoms.calc = SinglePointCalculator(atoms,energy=self.ir.E)
+          
           his.write(atoms=atoms)
           images.append(atoms)
           bonds   = getBonds(self.natom,self.ir.r,self.rmax*self.ir.re,self.ir.bo0)
@@ -642,6 +661,57 @@ class AtomDance(object):
       if reset: images.append(atoms_)
       return images,relaxlog
 
+  def get_zmat_uncertainty(self,atoms,optgen):
+      # zmats_init = np.array(self.InitZmat)
+      self.zmatrix  = np.array(self.get_zmatrix(atoms))
+      
+      for j in range(1):
+          for i,jj in enumerate(self.zmat_index):
+              if jj[j]>=0:
+                 iatom  = self.zmat_id[i]
+                 jatom  = jj[0]
+                 katom  = jj[1]
+                 latom  = jj[2]
+                 iatom_ = self.atom_name[iatom]
+                 jatom_ = self.atom_name[jatom]
+                 katom_ = self.atom_name[katom]
+                 latom_ = self.atom_name[latom]
+                 if j ==0:
+                   variable = iatom_+'-'+jatom_
+                 elif j==1:
+                   variable = iatom_+'-'+jatom_+'-'+katom_
+                 else:
+                   variable = iatom_+'-'+jatom_+'-'+katom_+'-'+latom_
+                 log,v  = self.get_optimal_zv(atoms,(i,j),optgen=optgen)
+                 r_     = self.InitZmat[i][j]
+                 rc     = self.zmatrix[i][j]
+                 self.InitZmat[i][j] = v
+                 if rc >=r_*1.1:
+                    lo = -r_*0.2
+                    hi = 0.0
+                 elif rc <=r_*0.9:
+                    lo = 0.0
+                    hi = r_*0.2
+                 else:
+                    lo = -r_*0.1
+                    hi = r_*0.1
+                 log    = '\n      --------------------------------------------------------\n' + log 
+                 if v>r_*1.07:   
+                    
+                    log  += '\nZvairiable {:s} from searching: {:8.4f}, bigger than uncertainty limit {:8.4f} ~ {:8.4f}\n'.format(variable,
+                            v,r_,r_*1.07)
+                    log  += '      --------------------------------------------------------\n'
+                    log  += '\n'
+                    return (i,j),lo,hi,log
+                 elif v<r_*0.93:
+                    # log  += '\nZvairiable {:s} from last searching: {:8.4f} \n'.format(variable,r_) 
+                    log  += '\nZvairiable {:s} from searching: {:8.4f}, smaller than uncertainty limit {:8.4f} ~ {:8.4f}\n'.format(variable,
+                             v,r_*0.93,r_)
+                    log  += '\n      --------------------------------------------------------\n'
+                    log  += '\n'
+                    return (i,j),lo,hi,log
+      return None,0.0,0.0,log
+
   def get_optimal_zv(self,atoms,zmat_variable,optgen=None,nbin=100,maxiter=100,
                        plot_opt=False):
       i,j           = zmat_variable
@@ -652,7 +722,7 @@ class AtomDance(object):
          zvlo,zvhi = -3.0,3.0
 
       bin_          = (zvhi-zvlo)/nbin
-      zmatrix       = self.InitZmat # np.array(self.get_zmatrix(atoms_))
+      zmatrix       = self.InitZmat.copy() # np.array(self.get_zmatrix(atoms_))
       initz         = zmatrix.copy()
       initz[i][j]   = zmatrix[i][j] + zvlo
       zmatrix[i][j] = zmatrix[i][j] + zvhi
@@ -662,7 +732,7 @@ class AtomDance(object):
       dEdz          = 1.0
       E             = []
       dz            = (zmatrix-initz)/nbin
-      optlog        = 'Getting the optimal zmatrix variable value ... \n'
+      optlog        = '\nSearching the optimal zmatrix variable value ... \n'
       iter_         = 0
 
       while abs(dEdz)>0.001 and iter_<maxiter:
@@ -695,17 +765,16 @@ class AtomDance(object):
             iter_ += 1
       result = half_[i][j]
       if iter_<maxiter:
-         optlog += '                     ---------------------------\n'
+         
          optlog += '                         %d-%d-%d-%d (%d,%d) \n' %(self.zmat_id[i],
                           self.zmat_index[i][0],self.zmat_index[i][1],self.zmat_index[i][2],i,j)
-         optlog += '                     ---------------------------\n'
-         optlog += 'The optimal variable value of (%d,%d) is %f ...\n' %(i,j,half_[i][j])
+         optlog += '\nThe optimal variable value of (%d,%d) is %f ...\n' %(i,j,half_[i][j])
          i_ = self.zmat_id[i]
          j_ = self.zmat_index[i][0]
          if j==0:
             if half_[i][j]>self.ir.re[i_][j_]*1.15 or half_[i][j]<self.ir.re[i_][j_]*0.8:
                half_[i][j] = self.ir.re[i_][j_]
-               optlog += 'The optimal of (%d,%d) exceed limmit, reset to preset value ...\n' %(i,j)
+               optlog += '\nThe optimal of (%d,%d) exceed limmit, reset to preset value ...\n' %(i,j)
          elif j==1:
             nn = len(self.neighbors[j_])
             if nn==3 or nn==4:
@@ -716,14 +785,14 @@ class AtomDance(object):
                da_ = abs(half_[i][j]-ang_)
                if da_ >5.0 :
                   half_[i][j] = ang_
-                  optlog += 'The optimal of (%d,%d) exceed limmit, reset to preset value ...\n' %(i,j)
-         self.InitZmat = half_
+                  optlog += '\nThe optimal of (%d,%d) exceed limmit, reset to preset value ...\n' %(i,j)
+         # self.InitZmat = half_
          self.write_zmat(half_,zfile='optimal.zmat')
          atoms_ = self.zmat_to_cartation(atoms_,half_)
          if optgen is not None: atoms_.write(optgen)
          del atoms_
       else:
-         optlog += 'The optimal variable search failed! \n'
+         optlog += '\nThe optimal variable search failed! \n'
          # print(zmat_variable,zvlo,zvhi)
 
       if plot_opt:
@@ -750,22 +819,6 @@ class AtomDance(object):
       e_ = self.ir.E
       dEdz   = (e_-e)/bin_
       return dEdz
-      
-  def get_zmat_uncertainty(self,atoms):
-      for i,jj in enumerate(self.zmat_index):
-         if jj[0]>=0:
-            iatom = self.zmat_id[i]
-            jatom = jj[0]
-            katom = jj[1]
-            latom = jj[2]
-            log,v = self.get_optimal_zv(atoms,(i,0))
-            if v>self.InitZmat[i][0]*1.15:   
-               # print('bond {:3d} {:2s} - {:3d} {:2s}:'.format(iatom,
-               #       self.atom_name[iatom],jatom,self.atom_name[jatom]),v)
-               return (i,0),self.InitZmat[i][0]*0.9,self.InitZmat[i][0]*1.2
-            elif v<self.ir.re[iatom][jatom]*0.9:
-               return (i,0),self.InitZmat[i][0]*0.8,self.InitZmat[i][0]*1.10
-      return None,0.0,0.0
 
   def get_zmat_info(self,zmats):
       if zmats is None or len(zmats)==0:
@@ -862,7 +915,6 @@ class AtomDance(object):
              return newbond,bd 
       return newbond,bd
 
-
   def check(self,wcheck=2,i=0,atoms=None,rmin=None):
       if atoms is None:
          atoms = self.ir.atoms
@@ -982,7 +1034,6 @@ class AtomDance(object):
       images = self.rotate(atms=[i,k],axis=axis,o=j,rang=rang,nbin=nbin,traj=traj,scale=scale)
       return images
 
-
   def bend_axis(self,axis=None,group=None,rang=20,nbin=30,scale=1.2,traj=None):
       images = self.rotate(atms=group,axis=axis,o=axis[0],rang=rang,nbin=nbin,traj=traj,scale=scale)
       return images
@@ -1027,13 +1078,20 @@ class AtomDance(object):
       da = 2.0*rang/nbin
       atoms = self.ir.atoms
       self.ir.calculate_Delta(atoms)
-      # neighbors = getNeighbor(self.natom,self.ir.r,scale*self.ir.re,self.ir.bo0)
+      if axis is not None:
+         i,j   = axis
+      if atms is None:
+         neighbors = getNeighbor(self.natom,self.ir.r,scale*self.ir.re,self.ir.bo0)
+         atms,ring = getAtomsToMove(i,j,j,[],neighbors)
+         if ring: atms = [j]
+         for a in atms:
+             if a in axis:
+                atms.remove(a)
 
       images = []
       if not traj is None: his = TrajectoryWriter(traj,mode='w')
 
       if axis_vector is None:
-         i,j   = axis
          vaxis = atoms.positions[j] - atoms.positions[i] 
          uk    = vaxis/self.ir.r[i][j]
       else:
@@ -1264,7 +1322,7 @@ class AtomDance(object):
           if not traj is None: his.write(atoms=atoms_)
       return images
 
-  def write_zmat(self,zmatrix,zfile=None,uspex=False):
+  def write_zmat(self,zmatrix,zfile='zmat',uspex=False):
       if zfile is None:
          zfile = self.label+'.zmat'
       
