@@ -1,5 +1,7 @@
 from os.path import exists,isfile
-from os import getcwd,chdir,mkdir,listdir,popen #,system,
+from os import getcwd,chdir,mkdir,listdir,system
+import numpy as np
+import json as js
 from ase.io import read,write
 from ase.io.trajectory import Trajectory
 from ase.constraints import FixAtoms
@@ -10,18 +12,15 @@ from .dft.siesta import siesta_md # siesta_opt
 from .dft.qe import qemd # qeopt  
 from .dft.SinglePointEnergy import SinglePointEnergies
 from .dft.CheckEmol import check_emol
-from .md.gulp import nvt as gulp_nvt
-from .data.mdtodata import MDtoData
+#from .md.gulp import nvt as gulp_nvt
+#from .data.mdtodata import MDtoData
 from .data.prep_data import prep_data
-#from .reaxfflib import read_ffield
 import tensorflow as tf
 from .md.irmd import IRMD
 from .trainer import train_reax
 from .trainer import train_mpnn
 from .AtomDance import AtomDance,check_zmat
 from .dingtalk import send_msg
-import numpy as np
-import json as js
 
 
 def plot_energies(it,edft,eamp,label_dft='SIESTA',label_ml='IRFF'):
@@ -43,7 +42,6 @@ def plot_energies(it,edft,eamp,label_dft='SIESTA',label_ml='IRFF'):
     plt.savefig('energies.pdf')                      # transparent=True
     plt.close() 
 
-
 def get_zpe(p,atom_name):
     zpe_ = 0.0
     for atom in atom_name:
@@ -71,15 +69,15 @@ class LearningMachine(object):
                learnWay=5,FreePairs=None,beta=None,
                FreeAtoms=None,FirstAtom=None,
                rodic=None,
-               rmin=0.6,rmax=1.3,angmax=15.0,
+               rmin=0.88,rmax=1.33,angmax=25.0,
                CheckZmat=True,
                EngTole=0.05,dEtole=0.2,dEstop=2.0,
                nn=True,vdwnn=False,
-               bo_layer=[2,1],mf_layer=[9,2],be_layer=[6,1],vdw_layer=None,#[6,1],
-               be_univeral_nn='all',bo_univeral_nn='all',mf_univeral_nn='all',vdw_univeral_nn='all',
-               BOFunction=0,EnergyFunction=1,MessageFunction=1,VdwFunction=1,
+               bo_layer=None,mf_layer=[9,1],be_layer=[9,1],vdw_layer=None,#[6,1],
+               be_universal_nn=None,bo_universal_nn=None,mf_universal_nn=None,vdw_universal_nn=None,
+               BOFunction=0,EnergyFunction=1,MessageFunction=3,VdwFunction=1,
                spv_be=False,beup={},belo={},
-               spv_bo=False,#boc={},# bolo={},
+               spv_bo=False,
                spv_vdw=False,vup={},vlo={},
                spv_ang=False,lambda_ang=0.02,
                weight={'others':2.0},
@@ -100,9 +98,9 @@ class LearningMachine(object):
       self.lossTole       = lossTole    
       self.accTole        = accTole      # Minimum accuracy convergence after a training process
       self.accMax         = accMax 
-      self.accInc         = accInc      # accuracy inrease factor
-      self.resetAcc       = resetAcc    # reset accuracy criteria after this train cycle
-      self.rmin           = rmin       # minimum bond length used to supervise ML-MD simulation
+      self.accInc         = accInc       # accuracy inrease factor
+      self.resetAcc       = resetAcc     # reset accuracy criteria after this train cycle
+      self.rmin           = rmin         # minimum bond length used to supervise ML-MD simulation
       self.rmax           = rmax
       self.angmax         = angmax
       self.CheckZmat      = CheckZmat
@@ -151,10 +149,10 @@ class LearningMachine(object):
       self.mf_layer       = mf_layer
       self.be_layer       = be_layer
       self.vdw_layer      = vdw_layer
-      self.bo_univeral_nn = bo_univeral_nn
-      self.be_univeral_nn = be_univeral_nn
-      self.mf_univeral_nn = mf_univeral_nn
-      self.vdw_univeral_nn= vdw_univeral_nn
+      self.bo_universal_nn = bo_universal_nn
+      self.be_universal_nn = be_universal_nn
+      self.mf_universal_nn = mf_universal_nn
+      self.vdw_universal_nn= vdw_universal_nn
       self.beup           = beup
       self.belo           = belo
       #self.boc            = boc
@@ -185,8 +183,8 @@ class LearningMachine(object):
       if self.freepairs is None:
          self.a.get_freebond(freeatoms=self.freeatoms)
       else:
-      	 self.a.freebonds = self.freepairs
-      	 
+         self.a.freebonds = self.freepairs
+ 
       self.natom        = self.a.natom
       self.atom_name    = self.a.atom_name
       self.get_ro(rodic)
@@ -227,7 +225,7 @@ class LearningMachine(object):
       e_gulp   = []
       e_siesta = []
       cwd      = getcwd()
-      gen      = 'poscar.gen'
+      # gen    = 'poscar.gen'
 
       if self.dataset is None:
          self.dataset = {}
@@ -253,13 +251,13 @@ class LearningMachine(object):
       self.save_config()
       converg        = False
       nconvergence   = 0
-      uncertainty_zv,uzv_ = None,None
+      uncertainty_zv = None
       zmat_variable  = None
       zmatopt        = None
-      zmats          = None
       images         = None
       zvlo,zvhi      = 0.0,0.0
       u_zvlo,u_zvhi  = 0.0,0.0
+      relax_step     = 5*self.colmin
 
       while running:
           optlog   = None
@@ -325,14 +323,15 @@ class LearningMachine(object):
           # atoms.set_constraint(self.fixatoms)
           # atoms.write('poscar.gen')                                    # for aimd
 
-          if learnWay==1 or (uncertainty_zv is not None): 
+          if learnWay==1: 
              dft_step = self.dft_step
+          elif Deformed>=1.0 or uncertainty_zv is not None:
+             dft_step = relax_step
           else:
              dft_step = int(mdsteps/self.colFrame)+1
-
           
           e_aimd,eml_,dEmax_,d2Emax_,LabelDataLog = self.aimd(atoms,cwd,run_dir,dft_step,learnWay)
-          e_siesta.append(e_aimd[0])                                # get siesta results              
+          e_siesta.append(e_aimd[0])                                   # get siesta results              
                                                                        # start training  
           trajs_ = prep_data(label=self.label,direcs=data_dir,
                              split_batch=self.batch,max_batch=self.max_batch,
@@ -365,10 +364,10 @@ class LearningMachine(object):
                                                      be_layer=self.be_layer,
                                                      vdw_layer=self.vdw_layer,
                                                      VdwFunction=self.VdwFunction,
-                                                     bo_univeral_nn=self.bo_univeral_nn,
-                                                     be_univeral_nn=self.be_univeral_nn,
-                                                     mf_univeral_nn=self.mf_univeral_nn,
-                                                     vdw_univeral_nn=self.vdw_univeral_nn,
+                                                     bo_universal_nn=self.bo_universal_nn,
+                                                     be_universal_nn=self.be_universal_nn,
+                                                     mf_universal_nn=self.mf_universal_nn,
+                                                     vdw_universal_nn=self.vdw_universal_nn,
                                                      BOFunction=self.BOFunction,
                                                      EnergyFunction=self.EnergyFunction,
                                                      MessageFunction=self.MessageFunction,
@@ -383,7 +382,7 @@ class LearningMachine(object):
                                                      learning_rate=self.learning_rate)
                 self.load_config()
                 if tIter>=self.resetAcc and tstep>=self.step:
-                   if accu>self.accCriteria-self.accTole and loss<self.lossCriteria+self.lossTole:
+                   if loss<self.lossCriteria+self.lossTole:   # accu>self.accCriteria-self.accTole and
                       self.accCriteria = float(accMax)
                       loss_ = float(loss)
                       if self.lossCriteria>loss_:
@@ -396,17 +395,17 @@ class LearningMachine(object):
           if self.lossCriteria>loss_:
              self.lossCriteria = loss_ 
           # self.lossCriteria  = float(loss)
-          
+
           if self.CheckZmat:
-             if not zmatopt is None:
-                optlog,_ = self.a.get_optimal_zv(atoms,zmatopt,optgen=self.initConfig)
+             # if not zmatopt is None:
+             #    optlog,_ = self.a.get_optimal_zv(atoms,zmatopt,optgen=self.initConfig)
              if mdsteps>10:
-                uncertainty_zv,u_zvlo,u_zvhi = self.a.get_zmat_uncertainty(atoms)
+                uncertainty_zv,u_zvlo,u_zvhi,optlog = self.a.get_zmat_uncertainty(atoms,optgen=self.initConfig)
 
           if mdsteps<10: mdsteps = 10
-          zmats   = None
-          images  = None
-          zmatopt = zmat_variable
+          # zmats    = None
+          images     = None
+          # zmatopt  = zmat_vairable # if Deformed>=1.0 else uncertainty_zv
 
           if learnWay==6:
              if ms<len(self.freeatoms):
@@ -416,24 +415,26 @@ class LearningMachine(object):
                 self.learnWay=3
           else:
              if Deformed>=1.0:
-                images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=zmat_variable,nbin=mdsteps,
+                images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=zmat_variable,nbin=relax_step,
                                                     zvlo=zvlo,zvhi=zvhi,traj='md.traj')
-             elif not uncertainty_zv is None:
-                images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=uncertainty_zv,nbin=mdsteps,
-                                                    zvlo=u_zvlo,zvhi=u_zvhi,traj='md.traj',reset=True)
+             elif uncertainty_zv is not None:
+                images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=uncertainty_zv,nbin=relax_step,
+                                                    zmatrix=self.a.zmatrix,zvlo=u_zvlo,zvhi=u_zvhi,traj='md.traj') # ,reset=True
              else:
-                e_gmd,mdsteps,Deformed,zmats,zmat_variable,zvlo,zvhi = self.mlmd(cwd,run_dir,atoms, 10000,
+                e_gmd,mdsteps,Deformed,zmat_variable,zvlo,zvhi = self.mlmd(atoms, 10000,
                                                 iter_,learnWay,beta=self.beta,learnpair=self.learnpair,
                                                 groupi=groupi,groupj=groupj) 
                 if self.CheckZmat and mdsteps<=3:
                    mdsteps= 10
                    images,relaxlog = self.a.continous(atoms,nbin=mdsteps,traj='md.traj')
-             if not images is None:
+             if images is not None:
                 Deformed,zmat,zmat_variable,zvlo,zvhi = check_zmat(atoms=images[-1],rmin=self.rmin,
                                        rmax=self.rmax,angmax=self.angmax,zmat_id=self.a.zmat_id,
                                        zmat_index=self.a.zmat_index,InitZmat=self.a.InitZmat)
-
-          step += mdsteps                                          # run ML-MD test training results     
+          if relax_step is None:        # 
+             step += mdsteps            # run ML-MD test training results     
+          else:                         # relax or MD
+             step += relax_step         #
 
           if accu >= self.accCriteria and tstep<self.step and tIter<=2: # get md steps
              if self.md_step<self.MaxMDstep and mdsteps>=self.md_step:
@@ -444,13 +445,9 @@ class LearningMachine(object):
              if self.md_step>self.MinMDstep:                                        # training results
                 self.md_step  = int(self.md_step/self.mdInc)            #                 
 
-          # if self.md_step>=self.MaxMDstep*0.5:
-          #    if self.T<self.Tmax:
-          #       self.T = self.T*self.mdInc
-
           self.save_config()
 
-          if learnWay in [3,4,5]: e_gmd = eml_
+          e_gmd = eml_ if learnWay in [3,4,5] else 0.0
           e_gulp.append(e_gmd[0])
           diff  = abs(e_gmd[0]-e_aimd[0])
           it.append(iter_)
@@ -458,7 +455,7 @@ class LearningMachine(object):
           plot_energies(it,e_siesta,e_gulp)                             # plot learning status                                   
           self.learninglog(iter_,loss,accu,mdsteps,e_gulp[-1],e_siesta[-1],dEmax_,d2Emax_,LabelDataLog,
           	               learnWay,Deformed,
-                           self.a.InitZmat,uncertainty_zv,u_zvlo,u_zvhi,optlog,relaxlog)
+                           self.a.InitZmat,uncertainty_zv,zvlo,zvhi,optlog,relaxlog)
           converg_ = converg
           if diff<=self.convergence: 
              converg = True
@@ -517,7 +514,7 @@ class LearningMachine(object):
             if Deformed>=1:
                l.write('Molecule structure is deformed, using zmatmix relax\n' )
             elif not uz is None:
-               l.write('Unreasonable zmatrix variable:\n')
+               l.write('Zmatrix variable:\n')
             l.write(relaxlog)
          if not optlog is None:
             l.write(optlog)
@@ -528,15 +525,17 @@ class LearningMachine(object):
       chdir(cwd+'/'+run_dir)
       
       if self.dft=='siesta':
-         popen('cp ../../*.psf ./')
+         system('cp ../../*.psf ./')
       elif self.dft=='qe':
-         popen('cp ../../*.UPF ./')
-
+         system('cp ../../*.UPF ./')
+      system('cp ../../ffield.json ./')     # prepare files
+      system('cp ../../md.traj ./')
+      
       emlmd  = []
       ind_   = ' '
       dEmax  = 0.0
       d2Emax = 0.0
-
+      
       if learnWay==1:
          if self.dft=='siesta':
             images = siesta_md(label=self.label,ncpu=self.ncpu,T=self.T,dt=self.dt_aimd,us='F',tstep=tstep,
@@ -551,19 +550,17 @@ class LearningMachine(object):
                           atoms=atoms,kpts=self.kpts,**self.kwargs)      ## geomentry optimize
             # images = qeopt(atoms=atoms,label=self.label,ncpu=self.ncpu,
             #                gen=cwd+'/'+gen,
-            #                kpts=self.kpts)         ## geomentry optimize
+            #                kpts=self.kpts,**self.kwargs) ## geomentry optimize
             ind_   = 'Geomentry Optimization'
          else:
             raise RuntimeError('-  This method not implimented!')
          eaimd = [images[0].get_potential_energy()]
-         popen('cp %s ../../md.traj' %(self.label+'.traj'))
+         system('cp %s ../../md.traj' %(self.label+'.traj'))
       elif learnWay>=2:
-         popen('cp ../../md.traj ./')
-         popen('cp ../../ffield.json ./')
-         # print(' * files in this dir \n',listdir())
-         E,E_,dEmax,d2Emax,ind_ = SinglePointEnergies('md.traj',label=self.label,EngTole=self.EngTole,
+         # print('-  files in this dir \n',cwd+'/'+run_dir,'\n',listdir())
+         E,E_,dEmax,d2Emax,ind_ = SinglePointEnergies(traj='md.traj',label=self.label,EngTole=self.EngTole,
                                                  frame=tstep,select=True,
-                                                 dE=self.dEtole, colmin=self.colmin,
+                                                 dE=self.dEtole,colmin=self.colmin,
                                                  dft=self.dft,kpts=self.kpts,
                                                  xcf=self.xcf,xca=self.xca,basistype=self.basistype,
                                                  cpu=self.ncpu,**self.kwargs)
@@ -574,10 +571,11 @@ class LearningMachine(object):
       chdir(cwd)
       return eaimd,emlmd,dEmax,d2Emax,ind_
 
-  def mlmd(self,cwd,run_dir,atoms,Tmax,Iter,learnWay,learnpair=None,beta=None,
+  def mlmd(self,atoms,Tmax,Iter,learnWay,learnpair=None,beta=None,
            groupi=None,groupj=None):        
       ''' run classic MD to test training results '''
       mdstep = max(int(self.md_step/5),2) if learnWay==2 else self.md_step
+      zmats  = None
       irmd = IRMD(atoms=atoms,label=self.label,Iter=Iter,initT=self.T,
                   time_step=self.dt_mlmd,totstep=mdstep,Tmax=Tmax,
                   ro=self.ro,rmin=self.rmin,rmax=self.rmax,angmax=self.angmax,
@@ -592,7 +590,7 @@ class LearningMachine(object):
       mdsteps= irmd.step
       Emd    = irmd.Epot
       irmd.close()
-      return Emd,mdsteps,Deformed,zmats,zv,zvlo,zvhi
+      return Emd,mdsteps,Deformed,zv,zvlo,zvhi
 
   def get_ro(self,rodic):
       if rodic is None:
@@ -638,18 +636,18 @@ class LearningMachine(object):
       else:
          consistance = False
       
-      cell  = atoms.get_cell()
-      array = cell.array
-      cell_ = self.a.atoms.get_cell()
-      array_= cell_.array
+      # cell  = atoms.get_cell()
+      # array = cell.array
+      # cell_ = self.a.atoms.get_cell()
+      # array_= cell_.array
 
-      for i,a in enumerate(array):
-          a_ = array_[i]
-          for j,x in enumerate(a):
-              x_ = a_[j]
-              # print(x,x_)
-              if abs(x-x_)>0.00001:
-                 consistance = False
+      # for i,a in enumerate(array):
+      #     a_ = array_[i]
+      #     for j,x in enumerate(a):
+      #         x_ = a_[j]
+      #         # print(x,x_)
+      #         if abs(1-x/x_)>0.2:
+      #            consistance = False
       return consistance
 
   def save_config(self):
@@ -769,12 +767,12 @@ if __name__ == '__main__':
                      EngTole=0.01,dEtole=0.05,dEstop=2.0,
                      EnergyFunction=1,
                      bo_layer=[4,1],mf_layer=[9,2],be_layer=[6,1],vdw_layer=[6,1],
-                     bo_univeral_nn=None,#['O-O'],
-                     be_univeral_nn=None,#['O-O'],
-                     mf_univeral_nn=None,#['C'],
+                     bo_universal_nn=None,#['O-O'],
+                     be_universal_nn=None,#['O-O'],
+                     mf_universal_nn=None,#['C'],
                      spv_be=False,bore={'others':0.22},
                      spv_ang=False,
-                     vdw_univeral_nn=None,vdwnn=True,
+                     vdw_universal_nn=None,vdwnn=True,
                      weight={# 'no2-l':20.0,
                              'others':2.0},
                      optword='nocoul-nolone-nounder-noover', 
