@@ -1,18 +1,15 @@
 from __future__ import print_function
-from os import system, getcwd, chdir,listdir
-from os.path import isfile,exists,isdir
+from os import getcwd, chdir,listdir
+from os.path import isfile
 from ase import Atoms
 from ase.io.trajectory import TrajectoryWriter,Trajectory
-from ase.calculators.siesta import Siesta
 from ase.calculators.singlepoint import SinglePointCalculator
-from ase.units import Ry
 from ..RadiusCutOff import setRcut
-import datetime
 import random
-import pickle
 import numpy as np
-import matplotlib.pyplot as plt
+#import matplotlib.pyplot as plt
 from ..molecule import press_mol
+#from ..dft.nwchem import out_xyz,get_nw_gradient
 
 
 def amp_data(lab='amp',
@@ -39,12 +36,12 @@ def amp_data(lab='amp',
                                      free_energy=float(md.max_e),
                                      forces=md.forces[i])
 
-        A.set_calculator(calc)
+        # A.set_calculator(calc)
+        A.calc = calc
         his.write(atoms=A)
         del A
     his.close()
 
-
 def get_lattice(inp='inp-nve'):
     ''' get cpmd lattice constance '''
     finp = open(inp,'r')
@@ -87,33 +84,31 @@ class MDtoData(object):
       self.sort      = sort
       self.checkMol  = checkMol
       self.structure = structure
-      self.batch = batch  
-      self.minib = minib    
-      self.qs    = None
+      self.batch     = batch  
+      self.minib     = minib    
+      self.qs        = None
 
       self.dft       = dft
       if direc.endswith('.traj'):
-         self.dft = 'ase'
+         self.dft    = 'ase'
 
       self.energy_nw,self.energy_bop = [],[]
-      self.table = []
+      self.table     = []
       self.atom_name = []
-      self.min_e = 0.0
-      self.max_e = 0.0
-      self.vdwcut = vdwcut
+      self.min_e     = 0.0
+      self.max_e     = 0.0
+      self.vdwcut    = vdwcut
 
       if p is None:
-         self.p  = {'vale_C':4,'vale_H':1,'vale_O':6,'vale_N':5,'vale_Al':3,'vale_F':7}
+         self.p   = {'vale_C':4,'vale_H':1,'vale_O':6,'vale_N':5,'vale_Al':3,'vale_F':7}
       else:
-         self.p  = p
+         self.p   = p
 
-      self.spec  = spec
+      self.spec   = spec
       
-      self.rcut  = rcut
-      self.rcuta = rcuta
-      self.traj  = traj
-      self.cell  = np.array([(10, 0, 0), (0, 10, 0), (0, 0, 10)])
-      self.status= True
+      self.rcut   = rcut
+      self.rcuta  = rcuta
+      self.traj   = traj
       self.initialize()
       self.rcut,self.rcuta,re = setRcut(self.bonds,rcut,rcuta,None)
       
@@ -137,12 +132,12 @@ class MDtoData(object):
          if self.dft!='ase':
             chdir(direc)
          
-      if self.dft=='nwchem':
-         outs = listdir(direc)
-         xs = self.get_nw_data(outs)
-      elif self.dft=='cpmd':
-         self.get_cpmd_frame()
-      elif self.dft=='siesta':
+      # if self.dft=='nwchem':
+      #    outs = listdir(direc)
+      #    xs = self.get_nw_data(outs)
+      # elif self.dft=='cpmd':
+      #    self.get_cpmd_frame()
+      if self.dft=='siesta':
          self.get_siesta_energy()
       elif self.dft=='qe':
          self.get_qe_energy()
@@ -170,9 +165,9 @@ class MDtoData(object):
          self.indexs = indexs[indices]
          forces,presses = None, None
 
-         if self.dft=='cpmd':
-            xs = self.get_cpmd_data()
-         elif self.dft=='siesta':
+         # if self.dft=='cpmd':
+         #    xs = self.get_cpmd_data()
+         if self.dft=='siesta':
             xs,cells = self.get_siesta_cart()
             forces,presses,qs = self.get_siesta_forces()
          elif self.dft=='ase':
@@ -213,9 +208,9 @@ class MDtoData(object):
             self.nframe=self.batch
          else:
             self.indexs = np.arange(self.nframe-nni)
-            if self.dft=='cpmd':
-               xs,forces = self.get_cpmd_data(indexs)
-            elif self.dft=='siesta':
+            # if self.dft=='cpmd':
+            #    xs,forces = self.get_cpmd_data(indexs)
+            if self.dft=='siesta':
                xs,cells    = self.get_siesta_cart()
                forces,presses,qs = self.get_siesta_forces()
             self.x = np.array(xs)
@@ -238,15 +233,13 @@ class MDtoData(object):
 
       self.min_e = min(self.energy_nw)
       self.max_e = max(self.energy_nw)
-      R,vr       = self.compute_bond(self.x,self.natom,self.nframe)
- 
-      self.vrs   = self.compute_image(vr)
+      # R,vr     = self.compute_bond(self.x,self.natom,self.nframe)
+      # self.vrs = self.compute_image(vr)
 
       if not direc is None:
          if self.dft!='ase':
             chdir(cdir)
 
-
   def initialize(self):
       self.bonds = []
       for sp1 in self.spec:
@@ -256,7 +249,6 @@ class MDtoData(object):
               if (not bd in self.bonds) and (not bdr in self.bonds):
                  self.bonds.append(bd)
 
-
   def init_names(self):
       self.bond_name= []
       for an1 in self.atom_name:
@@ -265,9 +257,7 @@ class MDtoData(object):
       # print(len(self.atom_name),self.atom_name)
       self.bond_name = np.reshape(self.bond_name,[self.natom,self.natom])
 
-
   def get_traj(self,inbox=False):
-      self.uc= np.linalg.inv(self.cell)
       images = []
       his    = TrajectoryWriter(self.structure+'.traj',mode='w')
       indexs = np.array(self.indexs)
@@ -276,10 +266,11 @@ class MDtoData(object):
 
       for i in range(batch_):
           ii  = ind[i]
+          uc  = np.linalg.inv(self.cells[ii])
           if inbox:
-             xf  = np.dot(self.x[ii],self.uc) 
+             xf  = np.dot(self.x[ii],uc) 
              xf  = np.mod(xf,1.0)  
-             self.x[ii] = np.dot(xf,self.cell)
+             self.x[ii] = np.dot(xf,self.cells[ii])
           if self.qs is None:
              c = None
           else:
@@ -298,9 +289,7 @@ class MDtoData(object):
       his.close()
       return images
 
-
   def get_images(self):
-      self.uc= np.linalg.inv(self.cell)
       images = []
       indexs = np.array(self.indexs)
       ind    = indexs.argsort()
@@ -308,9 +297,10 @@ class MDtoData(object):
       
       for i in range(batch_):
           ii  = ind[i]
-          xf  = np.dot(self.x[ii],self.uc) 
+          uc  = np.linalg.inv(self.cells[ii])
+          xf  = np.dot(self.x[ii],uc) 
           xf  = np.mod(xf,1.0)  
-          self.x[ii] = np.dot(xf,self.cell)
+          self.x[ii] = np.dot(xf,self.cells[ii])
 
           if self.qs is None:
              c = None
@@ -326,22 +316,20 @@ class MDtoData(object):
           images.append(A)
       return images
 
-
   def get_ase_energy(self,direc):
       images = Trajectory(direc)
       self.nframe = len(images)
       # print(images)
       return images
 
-
   def get_ase_data(self,images,trajonly):
-      ''' getting data in the ase traj '''
-      self.x=[]
-      self.energy_nw = []
+      ''' getting data from the ase traj '''
+      x         = []
+      energy_nw = []
+      cells     = []
       for i,ind_ in enumerate(self.indexs):
           imag = images[ind_]
           if i==0:
-             self.cell      =imag.get_cell()
              self.atom_name = imag.get_chemical_symbols()
              self.natom     = len(self.atom_name)
 
@@ -349,152 +337,12 @@ class MDtoData(object):
              e = 0.0
           else:
              e = imag.get_potential_energy()
-
-          self.x.append(imag.positions)
-          self.energy_nw.append(e)
-      self.energy_nw = np.array(self.energy_nw)
-      self.x = np.array(self.x)
-
-
-  def get_cpmd_frame(self):
-      if self.sort:
-         self.sort_data()
-      fe = open('ENERGIES','r')
-      lines = fe.readlines()
-      fe.close()
-      self.nframe = len(lines)
-
-
-  def get_cpmd_data(self):
-      fe = open('ENERGIES','r')
-      lines = fe.readlines()
-      fe.close()
-      nl = len(lines)
-      xs,gs = [],[]
-      self.energy_nw = []
-      if isfile('inp-nve'):
-         inp='inp-nve'
-      elif isfile('inp-nvt'):
-         inp='inp-nvt'
-      else: 
-         print('*  error: inp-nvt or inp-nve file not found!')
-         exit() 
-
-      cell      = get_lattice(inp=inp)  # surpose the cell is orthogonal
-      self.cell = np.array(cell)
-
-      for nf in range(nl):
-          l = lines[nf].split()
-          if len(l)>0:
-             self.energy_nw.append(float(l[3])*27.211396)  # to eV
-
-      fg = open('GEOMETRY.xyz','r')
-      lines = fg.readlines()
-      fg.close()
-      self.natom = len(lines) - 2
-      for na in range(2,self.natom+2):
-          self.atom_name.append(lines[na].split()[0])
-       
-      if isfile('FTRAJECTORY'):
-         ft = open('FTRAJECTORY','r')
-      else:
-         print('*  warning: forces not found ...')
-         ft = open('TRAJECTORY','r')
-      lines = ft.readlines()
-      ft.close()
-
-      ii = 0
-      au2ev = 27.211396/0.52917721067
-      for nf in range(nl):
-          x = []
-          g = []
-          for na in range(self.natom):
-              l = lines[nf*self.natom+na].split()
-              x.append([float(l[1])*0.52917721067,float(l[2])*0.52917721067,float(l[3])*0.52917721067])
-              if len(l)==10:
-                 # g.append([float(l[7])/0.52917721067,float(l[8])/0.52917721067,float(l[9])/0.52917721067])
-                 g.append([float(l[7])*au2ev,float(l[8])*au2ev,float(l[9])*au2ev])
-          xs.append(x)
-          if len(l)==10:
-             gs.append(g)
-      gs = np.array(gs)
-      return xs,gs
-
-
-  def sort_data(self):
-      ''' for cpmd data '''
-      system('mv ENERGIES ENERGIES.orig')
-      system('mv TRAJECTORY TRAJECTORY.orig')
-      system('mv FTRAJECTORY FTRAJECTORY.orig')
-
-      ft = open('FTRAJECTORY.orig','r')
-      fto = open('FTRAJECTORY','w')
-      first = None
-      for il,line in enumerate(ft.readlines()):
-          if line.find('<<<<<<  NEW DATA  >>>>>>') >0:
-             fto.close()
-             fto = open('FTRAJECTORY','w')
-             first = None
-          else:
-             if first is None:
-                first = line.split()[0]
-             print(line[:-1],file=fto)
-      fto.close()
-      ft.close()
-
-      ft = open('TRAJECTORY.orig','r')
-      fto = open('TRAJECTORY','w')
-      first = None
-      for il,line in enumerate(ft.readlines()):
-          if line.find('<<<<<<  NEW DATA  >>>>>>') >0:
-             fto.close()
-             fto = open('TRAJECTORY','w')
-             first = None
-          else:
-             if first is None:
-                first = line.split()[0]
-             print(line[:-1],file=fto)
-      fto.close()
-      ft.close()
-
-      fe = open('ENERGIES.orig','r')
-      feo = open('ENERGIES','w')
-      for il,line in enumerate(fe.readlines()):
-          if len(line.split())==0:
-             continue
-          if line.split()[0] == first:
-             if il>0:
-                feo.close()
-                # print(line[:-1])
-                feo = open('ENERGIES','w')
-             print(line[:-1],file=feo)
-          else:
-             print(line[:-1],file=feo)
-      feo.close()
-      fe.close()
-
-
-  def get_nw_data(self,outs):
-      i,ii = 0,0
-      xs = []
-      for out in outs:
-          if out.find('.out')>=0:
-             print('*  Getting molecular informations and nwchem datas from %s' %out)
-             natm,atoms,X = out_xyz(out_file=out)
-
-             xyz = out[:-4] + '.xyz'
-             e, gradient = get_nw_gradient(out)
-
-             if e is None or gradient is None:
-                continue
-             xs.append(X)
-             self.energy_nw.append(e)
-             ii += 1
-      self.nframe = ii
-      self.natom = natm
-      self.atom_name = atoms
-      return xs
-
+          cells.append(imag.get_cell())
+          x.append(imag.positions)
+          energy_nw.append(e)
+      self.cells     = np.array(cells)
+      self.energy_nw = np.array(energy_nw)
+      self.x         = np.array(x)
 
   def get_siesta_energy(self,label='siesta'):
       if isfile(label+'.MDE'):
@@ -517,7 +365,6 @@ class MDtoData(object):
          self.energy_nw = [0.0]
          self.nframe = 1
 
-
   def get_qe_energy(self,label='pw'):
       if isfile(label+'.MDE'):
          fe = open(label+'.MDE','r')
@@ -539,9 +386,8 @@ class MDtoData(object):
          self.energy_nw = [0.0]
          self.nframe = 1
 
-
-  def get_siesta_cart(self,label='siesta'):
-      fin = open('in.fdf','r') 
+  def get_siesta_cart(self,fdf='in.fdf',label='siesta'):
+      fin = open(fdf,'r') 
       lines= fin.readlines()
       fin.close()           # getting informations from input file
 
@@ -576,10 +422,34 @@ class MDtoData(object):
           else:
              self.atom_name.append(sp[int(l[3])-1])
 
-      if not isfile(label+'.MD_CAR'):
-         return []
-         
-      fe = open(label+'.MD_CAR','r')
+      if isfile(label+'.MD_CAR'):
+         xs,cells = self.parse_mdcar(label+'.MD_CAR')
+      else:
+         xs,cells = self.parse_out(label+'.out')
+      return xs,cells
+
+  def parse_out(self,fout):
+      with open(fout,'r') as fo:
+           lines = fo.readlines()
+      xs     = []
+      cells  = []
+      for i,line in enumerate(lines):
+          if line.find('outcoor: Relaxed atomic coordinates (Ang)')>=0:
+             x    = []
+             for na in range(self.natom):
+                 cl     = lines[na+i+1].split()
+                 x.append([float(cl[0]),float(cl[1]),float(cl[2])])
+             xs.append(x)
+          elif line.find('outcell: Unit cell vectors (Ang)')>=0: 
+             cell = []
+             for na in range(3):
+                 cl     = lines[na+i+1].split()
+                 cell.append([float(cl[0]),float(cl[1]),float(cl[2])])
+             cells.append(cell)
+      return xs,cells
+
+  def parse_mdcar(self,mdcar):     
+      fe = open(mdcar,'r')
       lines = fe.readlines()
       fe.close()
       nl = len(lines)
@@ -604,7 +474,7 @@ class MDtoData(object):
           c = [float(lc[0]),float(lc[1]),float(lc[2])]
           x = []
           il= 0
-          for i,s in enumerate(nsp):
+          for s in nsp:
               for ns in range(s):
                   l = lines[nl+7+il].split()
                   xd = [float(l[0]),float(l[1]),float(l[2])]
@@ -622,11 +492,8 @@ class MDtoData(object):
                   il += 1
           xs.append(x)
           cells.append(np.array([a,b,c]))
-
-      self.cell=np.array([a,b,c])
       return xs,cells
 
-
   def get_siesta_forces(self,label='siesta'):
       fo = open(label+'.out','r') 
       lines= fo.readlines()
@@ -754,85 +621,7 @@ class MDtoData(object):
 
              iframe += 1
       return np.array(forces),np.array(presses),np.array(qs)
-
-
-  def compute_bond(self,x,natom,nframe):
-      hfcell = 0.5 
-      u    = np.linalg.inv(self.cell)
-      x    = np.array(x)   #  project to the fractional coordinate
-      xf   = np.dot(x,u) 
-
-      xj   = np.expand_dims(xf,axis=1)
-      xi   = np.expand_dims(xf,axis=2)
-      vr   =  xj - xi
-
-      lm   = np.where(vr-hfcell>0)
-      lp   = np.where(vr+hfcell<0)
-
-      while (lm[0].size!=0 or lm[1].size!=0 or lm[2].size!=0 or
-             lp[0].size!=0 or lp[1].size!=0 or lp[2].size!=0):
-          vr = np.where(vr-hfcell>0,vr-1.0,vr)
-          vr = np.where(vr+hfcell<0,vr+1.0,vr)     # apply pbc
-          lm = np.where(vr-hfcell>0)
-          lp = np.where(vr+hfcell<0)
-      
-      vr_ = np.dot(vr,self.cell) # convert to ordinary coordinate
-      R   = np.sqrt(np.sum(vr_*vr_,axis=3),dtype=np.float32)
-      return R,vr_
-
-
-  def compute_image(self,vr):
-      vr_ = []
-      for i in range(-1,2):
-          for j in range(-1,2):
-              for k in range(-1,2):
-                  cell = self.cell[0]*i + self.cell[1]*j+self.cell[2]*k
-                  vr_.append(vr+cell)
-      return vr_
-    
-
-  def pdf(self,bins=0.01,rcut=4.0,pdf_plot=True):
-      print('-  plotting bond pair distribution function ...')
-      binsl,hists  = {},{}
-      for bd in self.bonds:
-          b = bd.split('-')
-          bdr = b[0]+'-'+b[1]
-
-          natom1 = np.sum(np.where(self.atom_name==b[0]))
-          natom2 = np.sum(np.where(self.atom_name==b[1]))
-          s=0.5 if b[0]==b[1] else 1.0
-             
-          ind_ = np.where(np.logical_or(self.bond_name==bd,self.bond_name==bdr))
-          for i,vr in enumerate(self.vrs):
-              vr2 = np.square(vr)
-              R   = np.sqrt(np.sum(vr2,axis=3),dtype=np.float32)
-              R_  = R[:,ind_[0],ind_[1]] 
-              rbd_ = R_ if i==0 else np.concatenate([rbd_,R_],axis=1)
-
-          rbd_= np.reshape(rbd_,[-1])
-          id_ = np.where(rbd_>0.000001)
-          rbd_= rbd_[id_]
-
-          hist,bin_ = np.histogram(rbd_,range=(0.0,rcut),bins=100,density=True)
-          bin_ = bin_[:-1]
-
-          binsl[bd]  = bin_
-          hists[bd] = hist
-
-          if pdf_plot:
-             plt.figure()   
-             plt.ylabel('Pair Distribution Function')
-             plt.xlabel(r'Radius $(\AA)$')
-             plt.xlim(0,rcut)
-             plt.ylim(0,np.max(hist)+0.01)
-             # plt.hist(rbd_,bin_=32,density=1,alpha=0.01,label='%s' %bd)
-             plt.plot(bin_,hist,alpha=0.01,label='%s' %bd)
-             plt.legend(loc='best')
-             plt.savefig('%s_bh.eps' %bd) 
-             plt.close() 
-      return binsl,hists
          
-
   def close(self):
       self.energy_nw,self.energy_bop = None,None
       self.table     = None
@@ -843,7 +632,7 @@ class MDtoData(object):
       self.atom_name = None
 
       
-if __name__ == '__main__':
-   dic_list = {'nmmol':'/home/gfeng/cpmd/train/nmr/nm12'}
-   TrainPrepare(dic_list)
+# if __name__ == '__main__':
+#    dic_list = {'nmmol':'/home/gfeng/cpmd/train/nmr/nm12'}
+#    TrainPrepare(dic_list)
 
