@@ -1,17 +1,19 @@
 from .siesta import single_point as single_point_siesta
 from .qe import single_point as single_point_qe
-from ase.io import read,write
+# from ase.io import read,write
 from ase.io.trajectory import TrajectoryWriter,Trajectory
-from ase.calculators.singlepoint import SinglePointCalculator
-from ase.units import Ry
-from ase import Atoms
+# from ase.calculators.singlepoint import SinglePointCalculator
+# from ase.units import Ry
+# from ase import Atoms
 import numpy as np
 
 
-def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
+def SinglePointEnergies(traj='md.traj',label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
                         EngTole=0.0000001,frame=50,cpu=4,dft='siesta',kpts=(1,1,1),
                         dE=0.2,colmin=2,select=False,**kwargs):
-    ''' get single point energy and labeling data '''
+    ''' get single point energy and labeling data
+        frame: number of frame to be selcted from the traj file 
+    '''
     #  if 'd2E' in kwargs:
     #      d2E = kwargs['d2E']
     #  else:
@@ -102,7 +104,7 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
            E.append(e)
            E_.append(e_)
         except:
-           print('- current DFT SCF calculation is not converged, stopping the structure calculation ...')
+           print('-  current DFT SCF calculation is not converged, stopping the structure calculation ...')
            continue # break #
 
         diff_ = abs(e-e_)
