@@ -36,13 +36,13 @@ def single_point(atoms,id=0,kpts=(1,1,1),
         #    return None
     return atoms_
 
-def qemd(atoms=None,gen='poscar.gen',kpts=(1,1,1),
-         label='pw',ncpu=1,T=300,dt=0.1,tstep=10):
+def qemd(atoms=None,gen='poscar.gen',kpts=(1,1,1),coord='cart',
+         label='pw',ncpu=1,T=300,tstep=50,**kwargs):
     if atoms is None:
        atoms = read(gen)
     write_qe_in(atoms,ion_dynamics='bfgs',
-                calculation='md',T=300,
-                kpts=kpts,koffset=(0, 0, 0)) # for MD
+                calculation='md',T=T,coord=coord,
+                kpts=kpts,koffset=(0, 0, 0),nstep=tstep,**kwargs) # for MD
     system('mpirun -n %d pw.x<pw.in>pw.out' %ncpu)
 
     his = TrajectoryWriter(label+'.traj',mode='w')
@@ -54,13 +54,15 @@ def qemd(atoms=None,gen='poscar.gen',kpts=(1,1,1),
     his.close()
     return images
 
-def qeopt(atoms=None,gen='poscar.gen',kpts=(1,1,1),
-          label='pw',ncpu=1):
+def qeopt(atoms=None,gen='poscar.gen',kpts=(1,1,1),VariableCell=False,
+          label='pw',ncpu=1,**kwargs):
     if atoms is None:
        atoms = read(gen)
+    calc='vc-relax' if VariableCell else 'relax'
     write_qe_in(atoms,ion_dynamics='bfgs',
-                calculation='relax',
-                kpts=kpts,koffset=(0, 0, 0)) # for structure relax
+                calculation=calc,
+                kpts=kpts,koffset=(0, 0, 0),
+                **kwargs) 
     system('mpirun -n %d pw.x<pw.in>pw.out' %ncpu)
 
     his = TrajectoryWriter(label+'.traj',mode='w')
@@ -81,7 +83,7 @@ def pseudopotentials(specie):
 def write_qe_in(atoms,
                 kspacing=None, kpts=None,koffset=(0, 0, 0),
                 calculation=None,          # supported: 'md','relax'
-                ion_dynamics='bfgs',
+                ion_dynamics='bfgs',T=300,coord='cart',
                 nspin=1,fspin=None,**kwargs):
     ''' wtrite QE(PWscf) input'''
     constraint_mask = np.ones((len(atoms), 3), dtype='int')
@@ -91,20 +93,23 @@ def write_qe_in(atoms,
         elif isinstance(constraint, FixCartesian):
             constraint_mask[constraint.a] = constraint.mask
         else:
-            warnings.warn('Ignored unknown constraint {}'.format(constraint))
+            print('Warning: Ignored unknown constraint {}'.format(constraint))
 
     atomic_species       = {} # atoms.get_chemical_symbols()
     atomic_species_str   = []
     atomic_positions_str = []
+    atomic_positions_frac_str = []
     mag_str = []
 
     if 'magmoms' in kwargs:
        atoms.set_initial_magnetic_moments(magmoms=kwargs['magmoms'])
 
     if any(atoms.get_initial_magnetic_moments()):
-        if nspin == 1:
-           # Force spin on
-           nspin = 2
+       if nspin == 1:
+          nspin = 2     # Force spin on
+
+    cell      = atoms.get_cell()
+    u         = np.linalg.inv(cell)           
 
     if nspin == 2:
         # Spin on
@@ -142,10 +147,16 @@ def write_qe_in(atoms,
                 '{atom.symbol}{tidx} '
                 '{atom.x:.10f} {atom.y:.10f} {atom.z:.10f}'
                 '{mask}\n'.format(atom=atom, tidx=tidx, mask=mask))
+
+            x = np.dot([atom.x,atom.y,atom.z],u)         #  project to the fractional coordinate
+            atomic_positions_frac_str.append(
+                '{atom.symbol}{tidx} '
+                '{x[0]:.10f} {x[1]:.10f} {x[2]:.10f}'
+                '{mask}\n'.format(atom=atom, x=x,tidx=tidx, mask=mask))
     else:
         for atom in atoms:
             if atom.symbol not in atomic_species:
-               atomic_species[atom.symbol] = True  # just a placeholder
+               atomic_species[atom.symbol] = True        # just a placeholder
                atomic_species_str.append(
                    '{species} {mass} {pseudo}\n'.format(
                        species=atom.symbol, mass=atom.mass,
@@ -162,6 +173,11 @@ def write_qe_in(atoms,
                '{atom.symbol} '
                '{atom.x:.10f} {atom.y:.10f} {atom.z:.10f} '
                '{mask}\n'.format(atom=atom, mask=mask))
+            x = np.dot([atom.x,atom.y,atom.z],u)          #  project to the fractional coordinate
+            atomic_positions_frac_str.append(
+               '{atom.symbol} '
+               '{x[0]:.10f} {x[1]:.10f} {x[2]:.10f}'
+               '{mask}\n'.format(atom=atom,x=x,mask=mask))
     # print(mag_str)
     # Add computed parameters
     # different magnetisms means different types
@@ -248,8 +264,8 @@ def write_qe_in(atoms,
 
     if calculation == 'md':
        pwi.append('&IONS\n')
-       pwi.append('  ion_temperature = \'not_controlled\' \n')
-       pwi.append('  tempw           = \'{:.4f}\' \n'.format(T))
+       #pwi.append('  ion_temperature = \'not_controlled\' \n')
+       pwi.append('  tempw           = {:.4f} \n'.format(T))
        pwi.append('/\n')  # terminate section
        pwi.append('\n')
     elif calculation == 'relax':
@@ -257,29 +273,35 @@ def write_qe_in(atoms,
        pwi.append('  ion_dynamics    = \'bfgs\' \n')
        pwi.append('/\n')  # terminate section
        pwi.append('\n')
-    # pwi.append('&CELL\n')
-    # pwi.append('/\n')  # terminate section
-    # pwi.append('\n')
-    # True and False work here and will get converted by ':d' format
-    # KPOINTS - add a MP grid as required
+    elif calculation == 'vc-relax':
+       pwi.append('&IONS\n')
+       pwi.append('  ion_dynamics    = \'bfgs\' \n')
+       pwi.append('/\n')  # terminate section
+       pwi.append('\n')
+       pwi.append('&CELL\n')
+       pwi.append('  cell_dynamics    = \'bfgs\' \n')
+       pwi.append('/\n')  # terminate section
+       pwi.append('\n')
+    else:
+       raise RuntimeError('-  This type of run of QE is not supported yet!')
     
     if kspacing is not None:
-        kgrid = kspacing_to_grid(atoms, kspacing)
+       kgrid = kspacing_to_grid(atoms, kspacing)
     elif kpts is not None:
-        if isinstance(kpts, dict) and 'path' not in kpts:
-            kgrid, shift = kpts2sizeandoffsets(atoms=atoms, **kpts)
-            koffset = []
-            for i, x in enumerate(shift):
-                assert x == 0 or abs(x * kgrid[i] - 0.5) < 1e-14
-                koffset.append(0 if x == 0 else 1)
-        else:
-            kgrid = kpts
+       if isinstance(kpts, dict) and 'path' not in kpts:
+          kgrid, shift = kpts2sizeandoffsets(atoms=atoms, **kpts)
+          koffset = []
+          for i, x in enumerate(shift):
+              assert x == 0 or abs(x * kgrid[i] - 0.5) < 1e-14
+              koffset.append(0 if x == 0 else 1)
+       else:
+          kgrid = kpts
     else:
-        kgrid = (1, 1, 1)
+       kgrid = (1, 1, 1)
 
     # True and False work here and will get converted by ':d' format
     if isinstance(koffset, int):
-        koffset = (koffset, ) * 3
+       koffset = (koffset, ) * 3
 
     # CELL block, if required
     pwi.append('CELL_PARAMETERS angstrom\n')
@@ -295,14 +317,19 @@ def write_qe_in(atoms,
     pwi.append('\n')
 
     # Positions - already constructed, but must appear after namelist
-    pwi.append('ATOMIC_POSITIONS angstrom\n')
-    pwi.extend(atomic_positions_str)
+    if coord=='crystal':
+       pwi.append('ATOMIC_POSITIONS crystal\n')
+       pwi.extend(atomic_positions_frac_str)
+    else:
+       pwi.append('ATOMIC_POSITIONS angstrom\n')
+       pwi.extend(atomic_positions_str)
     pwi.append('\n')
 
     # BandPath object or bandpath-as-dictionary:
     if 'K_POINTS' in kwargs:
        pwi.append('K_POINTS automatic\n')
-       pwi.append(kwargs['K_POINTS'])
+       pwi.append('{0[0]} {0[1]} {0[2]}  {1[0]:d} {1[1]:d} {1[2]:d}\n'
+                  ''.format(kwargs['K_POINTS'], koffset))
     else:
        pwi.append('K_POINTS automatic\n')
        pwi.append('{0[0]} {0[1]} {0[2]}  {1[0]:d} {1[1]:d} {1[2]:d}\n'
