@@ -2,7 +2,8 @@ from os import system,getcwd
 from os.path import isfile,exists
 from ..molecule import enlarge,molecules,get_neighbors
 from ..data.mdtodata import MDtoData
-from ase.io import read,write
+from ase.io import read
+from ase.calculators.siesta import Siesta
 from ase.io.trajectory import TrajectoryWriter
 from ase.calculators.singlepoint import SinglePointCalculator
 from ase.units import Ry
@@ -79,11 +80,17 @@ def Basis():
                    ' n=3   2  1',
                    '   5.0574',
                    '   1.000'],
-               'Fe':['Fe  2 ',
-                     '0  2  P',
-                     '6. 0.',
-                     '2  2',
-                     '0. 0.']
+               'Fe':['Fe 5',
+                     ' n=3 0 2 E 40 -0.9',
+                     '   8.0 2.4446409',
+                     ' n=3 1 2 E 40 -0.9',
+                     '   8.0 2.2136430',
+                     ' n=3 2 2 E 40 -0.9',
+                     '   5.85260682156 3.0385538',
+                     ' n=4 0 2 E 40 -0.9',
+                     '   9.17876877610 7.1301390',
+                     ' n=4 1 1 E 40 -0.9 Q 1.3284039 19.9788401',
+                     '   9.17876877610']
                   }
     return basis
 
@@ -134,7 +141,8 @@ def single_point(atoms,id=0,xcf='VDW',xca='DRSLL',basistype='DZP',
     
     # atoms.set_initial_charges(charges=q[-1])
     calc = SinglePointCalculator(atoms,energy=e,forces=f[-1],stress=p[-1])
-    atoms.set_calculator(calc)
+    # atoms.set_calculator(calc)
+    atoms.calc = calc
     system('rm siesta.* ')
     return atoms
 
@@ -210,9 +218,12 @@ def write_siesta_in(Atoms,coord='zmatrix',
     r[2] = cell[2][0]*cell[2][0]+ cell[2][1]*cell[2][1]+ cell[2][2]*cell[2][2]
     r[2] = np.sqrt(r[2])
     
-    kgrid = min(r)
-    if singlek:
-       kgrid=0.0
+    if 'KgridCutoff' in kwargs: 
+       kgrid=kwargs['KgridCutoff']
+    else:
+       kgrid = min(r)
+       if singlek:
+          kgrid=0.0
 
     atoms_label = A.get_chemical_symbols()
     species     = []
@@ -242,20 +253,32 @@ def write_siesta_in(Atoms,coord='zmatrix',
     
     if 'MaxSCFIterations' in kwargs:
        print('MaxSCFIterations        {:d} \n'.format(kwargs['MaxSCFIterations']),file=fdf)
-    # print('PAO.EnergyShift  100 meV',file=fdf)
+    if 'MaxSCFIterations' in kwargs:
+       print('MaxSCFIterations        {:d} \n'.format(kwargs['MaxSCFIterations']),file=fdf)
+    if 'EnergyShift' in kwargs: 
+       print('PAO.EnergyShift  {:f} meV'.format(kwargs['EnergyShift']),file=fdf)
+
     
-    if basistype=='split':
+    if basistype=='split':                            ### setting basis set and size
        print('PAO.BasisType    split',file=fdf)
        print('%block PAO.Basis',file=fdf)
-       for i,spec in enumerate(species):
+       for spec in species:
            assert spec in basis
            for l in basis[spec]:
                print(l,file=fdf)
        print('%endblock PAO.Basis',file=fdf)
     else:
        print('PAO.BasisSize    %s     # standard basis set, Like DZ plus polarization' %basistype,file=fdf)
-    print('SpinPolarized    %s' %spin,file=fdf)
+       if 'Basis' in kwargs:
+          basis = kwargs['Basis']
+          print('%block PAO.Basis',file=fdf)
+          for spec in basis:
+              for l in basis[spec]:
+                  print(l,file=fdf)
+          print('%endblock PAO.Basis',file=fdf)      ### end setting Basis set and size
+    
     print('   ',file=fdf)
+    print('SpinPolarized    %s' %spin,file=fdf)
     print('   ',file=fdf)
 
     if 'InitSpinAF' in kwargs:
@@ -292,7 +315,10 @@ def write_siesta_in(Atoms,coord='zmatrix',
        else:
           print('MD.Steps         %d' %tstep,file=fdf)
           print('MD.MaxCGDispl    0.1 Ang ',file=fdf)
-       print('MD.MaxForceTol   0.03 eV/Ang ',file=fdf)
+       if 'MaxForceTol' in kwargs:
+          print('MD.MaxForceTol        {:f} eV/Ang '.format(kwargs['MaxForceTol']),file=fdf)
+       else:
+          print('MD.MaxForceTol   0.03 eV/Ang ',file=fdf)
        print('MD.UseSaveXV     %s ' %us,file=fdf) # T for restart
        print('MD.UseSaveZM     false ',file=fdf)
     print('   ',file=fdf)
@@ -306,7 +332,14 @@ def write_siesta_in(Atoms,coord='zmatrix',
        print('MeshCutoff        {:f} Ry \n'.format(kwargs['MeshCutoff']),file=fdf)
     else:
        print('MeshCutoff       200. Ry # Equivalent planewave cutoff for the grid ',file=fdf)
-    print('KgridCutoff      %f Ang' %kgrid,file=fdf)
+    if 'MonkhorstPack' in kwargs: # KgridCutoff
+       print('%block kgrid.MonkhorstPack',file=fdf)
+       print('{:d} 0 0 {:f}'.format(kwargs['MonkhorstPack'][0],kwargs['MonkhorstPack'][3]),file=fdf)
+       print('0 {:d} 0 {:f}'.format(kwargs['MonkhorstPack'][1],kwargs['MonkhorstPack'][3]),file=fdf)
+       print('0 0 {:d} {:f}'.format(kwargs['MonkhorstPack'][2],kwargs['MonkhorstPack'][3]),file=fdf)
+       print('%endblock kgrid.MonkhorstPack',file=fdf)
+    else:
+       print('KgridCutoff      %f Ang' %kgrid,file=fdf)
     print('   ',file=fdf)
     print('   ',file=fdf)
     print('WriteCoorInitial T',file=fdf)
@@ -323,7 +356,7 @@ def write_siesta_in(Atoms,coord='zmatrix',
     print(cell[2][0],cell[2][1],cell[2][2],file=fdf)
     print('%endblock LatticeVectors',file=fdf)
     print('   ',file=fdf)
-    
+
     if coord=='cart':
        print('AtomicCoordinatesFormat Ang',file=fdf)
        print('   ',file=fdf)
@@ -717,7 +750,8 @@ def get_siesta_cart(label='siesta'):
         xs.append(x)
 
         A = Atoms(atom_name,x,cell=[a,b,c],pbc=[True,True,True])
-        A.set_calculator(calc)
+        A.calc = calc
+        # A.set_calculator(calc)
         # A._calc.results['energy'] = 
         his.write(atoms=A)
 
@@ -753,6 +787,93 @@ def parse_fdf_species(fdf='in.fdf'):
                      spec[s+1] = l_[2].split('.')[0]
     return spec
 
+def parse_out(outfile='siesta.out',spec={}):
+    f  = open(outfile,'r') 
+    lines= f.readlines()
+    f.close()        
+    atom_name = []
+    natom     = 0
+    ns        = 0
+
+    for i,line in enumerate(lines):
+        l = line.split()
+        if len(l)>0:
+           if l[0] == 'NumberOfSpecies':
+              ns = int(l[1])
+           if l[0] == 'NumberOfAtoms':
+              natom = int(l[1])
+        if ns and natom:
+           break
+
+    positions = []
+    atom_name = []
+    cell      = []
+    pressure  = []
+    presses   = []
+    forces    = []
+    e = 0.0
+
+    for i,line in enumerate(lines):
+        l = line.split()
+        if len(l)>0:
+           if l[0]=='%block':
+              if l[1]=='LatticeVectors':
+                 for s in range(3):
+                     l_ = lines[i+s+1].split()
+                     cell.append([float(l_[0]),float(l_[1]),float(l_[2])])
+              if l[1]=='ChemicalSpeciesLabel' or l[1]=='ChemicalSpecieslabel':
+                 for s in range(ns):
+                     l_ = lines[i+s+1].split()
+                     spec[s+1] = l_[2].split('.')[0]
+              if l[1]=='AtomicCoordinatesAndAtomicSpecies':
+                 for p in range(natom):
+                     l_ = lines[i+p+1].split()
+                     positions.append([float(l_[0]),float(l_[1]),float(l_[2])])
+                     # print(spec,int(l_[3])-1)
+                     atom_number = int(l_[3])
+                     if atom_number not in spec:
+                        spec[atom_number] = input('Please input symbol of number {:d}: '.format(atom_number))
+                     atom_name.append(spec[atom_number])
+           elif l[0]=='LatticeConstant':
+              if line.find('Ang')>0 or line.find('ang')>0:
+                 unit = 1.0
+              elif line.find('Bohr')>0 or line.find('Bohr')>0:
+                 unit = 0.529177249
+              else:
+                 unit = 1.0
+
+           elif line.startswith('siesta: E_KS(eV)'):
+              e = float(line.split()[-1])
+           elif line.find('Atomic forces (eV/Ang)')>=0:
+              ln = lines[i+1]
+              force   = []
+              if ln.find('----------------------------------------')<0:
+                 for na in range(natom):
+                     fl = lines[na+i+1] #.split()
+                     if fl.find('siesta:')<0:
+                           f1 = fl[6:18]
+                           f2 = fl[18:30]
+                           f3 = fl[30:42]
+                           force.append([float(f1),float(f2),float(f3)])
+              if len(force)>0:
+                 forces.append(force)
+           elif line.find('Stress tensor (static)')>=0:
+              press   = []
+              for l in range(3):
+                  pl = lines[l+i+1].split()
+                  press.append([float(pl[1]),float(pl[2]),float(pl[3])])
+              presses.append(press)
+           elif line.find('Pressure (static):')>=0:
+              l = lines[i+4].split()
+              pressure.append(float(l[2]))
+
+    cell      = np.array(cell)*unit
+    positions = np.array(positions)*unit
+    atoms     = Atoms(atom_name,positions,cell=cell,pbc=[True,True,True])
+    calc      = SinglePointCalculator(atoms,energy=e,forces=forces[-1],stress=presses[-1])
+    atoms.calc = calc
+    return atoms 
+
 def parse_fdf(label='siesta',spec={}):
     fin = open(label+'.fdf','r') 
     lines= fin.readlines()
