@@ -1010,7 +1010,6 @@ class IRFF_NP(object):
       self.get_rcbo()
       self.set_p(m,self.bo_layer)
 
-
   def calculate_Delta(self,atoms=None,updateP=False):
       if updateP:
          self.update_p()
@@ -1023,15 +1022,7 @@ class IRFF_NP(object):
       xf        = np.dot(positions,rcell)
       xf        = np.mod(xf,1.0)
       positions = np.dot(xf,cell).astype(dtype=np.float32)
-
-      # self.get_charge(cell,positions)
-      self.get_neighbor(cell,rcell,positions)
       self.get_ebond(cell,rcell,positions)
-
-      n       = np.where(np.logical_and(self.r>0.0001,self.r<self.r_cuta),1.0,0.0)
-      N       = np.sum(n,axis=1)                      # nearest neighbor matrix
-      self.ND = N-self.P['val']                       # 
-
       
   def calculate(self,atoms=None):
       cell      = atoms.get_cell()                    # cell is object now
