@@ -45,20 +45,24 @@ def update_ffield(p_,ffield):
     with open('ffield.json','w') as fj:
          js.dump(j,fj,sort_keys=True,indent=2)
 
-
-def ffield_to_csv(ffield='ffield.json',parameters=None,fcsv='ffield.csv'):
+def ffield_to_csv(ffield='ffield.json',parameters=None,keys=None,fcsv='ffield.csv'):
     with open(ffield,'r') as lf:
-         j = js.load(lf)
+         j  = js.load(lf)
          p_ = j['p']
-         m_ = j['m']
-         bo_layer = j['bo_layer']
+         try:
+            score = j['score']
+         except:
+            score = -1000
+         # m_ = j['m']
+         # bo_layer = j['bo_layer']
     spec,bonds,offd,angs,torp,hbs = init_bonds(p_)
     # cons   = ['val','vale','lp3','cutoff','acut','hbtol']
     p_g    = ['boc1','boc2','coa2','ovun6','lp1', # 'lp3',
               'ovun7','ovun8','val6','val9','val10','tor2',
               'tor3','tor4','cot2','coa4','ovun4', 
-              'ovun3','val8','coa3','pen2','pen3','pen4','vdw1']
-              # 'cutoff','acut','hbtol'
+              'ovun3','val8','coa3','pen2','pen3','pen4','vdw1',
+              'cutoff','acut','hbtol']
+              # 
               
     p_spec = ['valboc','ovun5',
               'lp2','boc4','boc3','boc5','rosi','ropi','ropp',
@@ -134,6 +138,12 @@ def ffield_to_csv(ffield='ffield.json',parameters=None,fcsv='ffield.csv'):
                   
     p_name.append('score')
 
+    if keys is not None:
+       p_name = ['']
+       for key in keys:
+           if key:               # 过虑空列
+              p_name.append(key)
+
     already_exist = False
     if isfile(fcsv):
        already_exist = True
@@ -151,22 +161,20 @@ def ffield_to_csv(ffield='ffield.json',parameters=None,fcsv='ffield.csv'):
     for key in p_name:
         if key:
            if key=='score':
-              s = '{:.4f}'.format(float(-10000.0))
+              s = '{:.4f}'.format(score)
            else:
               s = '{:.4f}'.format(p_[key])
            row.append(s)
 
-    # row.append('{:.4f}'.format(-100000.0))
     csv_write.writerow(row)
     fcsv_.close()
-
+    return p_name,row
 
 def get_csv_data(fcsv):
     d = pd.read_csv(fcsv)
     # print(d['boc1'])
     return d
 
-
 def init_bonds(p_):
     spec,bonds,offd,angs,torp,hbs = [],[],[],[],[],[]
     for key in p_:
