@@ -1,19 +1,11 @@
 import matplotlib.pyplot as plt
 from os import makedirs
-from os.path import isfile,exists,isdir
-# from .reax_data import get_data 
-# from .link import links
+from os.path import exists
 from .reaxfflib import write_ffield,write_lib
 from .reax import ReaxFF,taper
-# from .initCheck import Init_Check
-# from .dingtalk import send_msg
-# import time
-# from ase import Atoms
-from ase.io.trajectory import Trajectory
 import tensorflow as tf
 # from tensorflow.contrib.opt import ScipyOptimizerInterface
 import numpy as np
-# import random
 import json as js
 # tf_upgrade_v2 --infile reax.py --outfile reax_v1.py
 # tf.compat.v1.disable_v2_behavior()
@@ -68,6 +60,52 @@ def fmessage(pre,bd,nbd,x,m,batch=50,layer=5):
     out= tf.reshape(o_,[nbd,batch,3])
     return out
 
+def dfmessage(pre,bd,nbd,x,m,batch=50,layer=5):
+    ''' 
+         input  X: (nbd*batch,3) 
+               Wi:  (3,9)  bi: (9,)
+               Wh:  (9,9)  bh: (9,)
+               Wo:  (9,3)  bo: (3,)
+           output: (nbd*batch,3)
+    '''
+    nd = len(x)
+    x_ = []
+    for d in x:
+        x_.append(tf.reshape(d,[nbd*batch]))
+    X  = tf.stack(x_,axis=1)     # (nbd*batch,3)  
+    X  = tf.expand_dims(X, axis=-2)      # expand dimension  (nbd*batch,1,3)
+
+    o  = [] 
+    z  = []
+
+    z.append(tf.matmul(X,m[pre+'wi_'+bd],name='bop_input')+m[pre+'bi_'+bd])  # input layer
+    o.append(tf.sigmoid(z[-1]))   
+
+    for l in range(layer):                                                   # hidden layer    
+        z.append(tf.matmul(o[-1],m[pre+'w_'+bd][l],name='bop_hide')+m[pre+'b_'+bd][l])  
+        o.append(tf.sigmoid(z[-1]))
+    
+    z.append(tf.matmul(o[-1],m[pre+'wo_'+bd],name='bop_output') + m[pre+'bo_'+bd])  # output layer
+    o_ = tf.sigmoid(z[-1])       
+    out= tf.reshape(o_,[nbd,batch,3])
+
+    ''' ------------- derivative of do_/dX ------------- '''
+    d  = []
+    
+    d.append(tf.transpose(m[pre+'wo_'+bd])*tf.transpose(dsigmoid(z[-1]),perm=(0,2,1)))  # output layer do_/do[-1]
+
+    for l in range(layer):
+        d.append(tf.transpose(m[pre+'w_'+bd][layer-l-1])*tf.transpose(dsigmoid(z[-l-2]),perm=(0,2,1)))  # hidden layer backprop
+
+    d.append(tf.transpose(m[pre+'wi_'+bd])*tf.transpose(dsigmoid(z[0]),perm=(0,2,1)))  # input layer do[0]/dX
+
+    df = d[0]
+    for l in range(layer):
+        df = tf.matmul(df,d[l+1])
+    df = tf.matmul(df,d[-1])
+    df = tf.reshape(df,[nbd,batch,3,3])
+    return df
+
 
 class MPNN(ReaxFF):
   def __init__(self,libfile='ffield',dataset={},
@@ -98,7 +136,7 @@ class MPNN(ReaxFF):
                vup={'others':[(10.0,0.0)]},
                pim={'others':10.0},
                spv_be=False,                             
-               spv_bo=None,                      # e.g. spv_bo={'C-C':(3.0,0.2,1.0)}
+               spv_bo=None,                      # e.g. spv_bo={'C-C':(1.3,0.2,1.0)}
                spv_pi=False,
                spv_ang=False,
                spv_vdw=False,
@@ -111,15 +149,14 @@ class MPNN(ReaxFF):
                resetDeadNeuron=False,
                messages=1,
                be_universal_nn=None,be_layer=[9,0],
-               bo_universal_nn=None,bo_layer=[6,0],
+               bo_universal_nn=None,bo_layer=None,   # [6,0]
                mf_universal_nn=None,mf_layer=[9,0],
-               vdw_universal_nn=None,vdw_layer=[9,0],
+               vdw_universal_nn=None,vdw_layer=None, # [9,0]
                vdwnn=False,VdwFunction=1,
                BOFunction=0,
                EnergyFunction=1,
                MessageFunction=3,
                spec=[],
-               sort=False,
                lambda_bd=100000.0,
                lambda_pi=1.0,
                lambda_reg=0.01,
@@ -197,7 +234,7 @@ class MPNN(ReaxFF):
                       clip_op=clip_op,clip=clip,
                       InitCheck=InitCheck,resetDeadNeuron=resetDeadNeuron,
                       nn=nn,vdwnn=vdwnn,VdwFunction=VdwFunction,
-                      bo_layer=bo_layer,spec=spec,sort=sort,weight=weight,
+                      bo_layer=bo_layer,spec=spec,weight=weight,
                       lambda_bd=lambda_bd,
                       optMethod=optMethod,maxstep=maxstep,
                       emse=emse,convergence=convergence,lossConvergence=lossConvergence,
@@ -250,6 +287,10 @@ class MPNN(ReaxFF):
           mols = mol.split('-')[0] 
           self.ME += tf.square(self.MolEnergy[mols])
 
+      if self.pseudo_data:
+         self.loss_pseudo   = self.pseudo()
+         self.Loss         += self.loss_pseudo
+
       self.loss_penalty  = self.supervise()
       self.Loss         += self.loss_penalty
 
@@ -726,6 +767,26 @@ class MPNN(ReaxFF):
                         self.vdw_layer,self.vdw_layer_,self.VdwFunction_,self.VdwFunction,
                         self.bo_universal_nn,self.be_universal_nn,
                         self.mf_universal_nn,self.vdw_universal_nn)
+  
+  def pseudo(self):
+      ''' training the neural network with pseudo data '''
+      loss = tf.constant(0.0)
+      for bd in self.bonds: 
+          Fi   = fmessage(flabel,b[0],self.nbd[bd],[self.Dbi[bd],h,self.Dbj[bd]],
+                          self.m,batch=self.batch,layer=self.mf_layer[1])
+          Fj   = fmessage(flabel,b[1],self.nbd[bd],[self.Dbj[bd],h,self.Dbi[bd]],
+                              self.m,batch=self.batch,layer=self.mf_layer[1])
+          F    = Fi*Fj
+          Fsi,Fpi,Fpp = tf.unstack(F,axis=2)
+
+          bosi = self.bopsi_pse[bd]*Fsi
+          bopi = self.boppi_pse[bd]*Fpi
+          bopp = self.boppp_pse[bd]*Fpp
+
+          loss += tf.nn.l2_loss(bosi-self.bosi_pse[bd])
+          loss += tf.nn.l2_loss(bopi-self.bopi_pse[bd])
+          loss += tf.nn.l2_loss(bopp-self.bopp_pse[bd])
+      return loss
 
   def supervise(self):
       ''' adding some penalty term to accelerate the training '''
@@ -746,8 +807,10 @@ class MPNN(ReaxFF):
 
       w_n     = ['wi','wo',]
       b_n     = ['bi','bo']
-      layer   = {'fe':self.be_layer[1],'fsi':self.bo_layer[1],'fpi':self.bo_layer[1],
-                 'fpp':self.bo_layer[1]}
+      layer   = {'fe':self.be_layer[1]}
+      if self.bo_layer is not None:
+         layer['fsi'] = layer['fpi'] = layer['fpp'] = self.bo_layer[1]
+                 
       if self.vdwnn:
          layer['fv'] = self.vdw_layer[1]
 
@@ -785,7 +848,6 @@ class MPNN(ReaxFF):
 
           if self.nbd[bd]>0:
              bd_ = bd  if bd in self.bore else 'others'
-
              fao = tf.where(tf.greater(self.rbd[bd],self.rcuta[bd]),1.0,0.0)    ##### r> rcuta that bo = 0.0
              self.penalty_bo_rcut[bd]  = tf.reduce_sum(self.bo0[bd]*fao)
              penalty = tf.add(self.penalty_bo_rcut[bd]*self.lambda_bd,penalty)
@@ -804,9 +866,13 @@ class MPNN(ReaxFF):
                    self.penalty_be[bd] = tf.constant(0.0)
                    if (bd in self.beup) or (bdr in self.beup):
                       bd_ = bd if bd in self.beup else bdr
-                      for beup_ in self.beup[bd_]:
-                          r_,Di_l,Di_u,Dj_l,Dj_u,be_u = beup_  
-                          fu = tf.where(tf.logical_and(tf.less_equal(self.rbd[bd],r_),
+                      for i,beup_ in enumerate(self.beup[bd_]):
+                          r_,Di_l,Di_u,Dj_l,Dj_u,be_u = beup_ 
+                          if i==0:
+                             rleft = r_
+                             continue
+                          fu = tf.where(tf.logical_and(tf.logical_and(tf.less_equal(self.rbd[bd],r_),
+                                                                      tf.greater(self.rbd[bd],rleft)),
                                              tf.logical_and(
                                                  tf.logical_and(tf.less_equal(self.Dbi[bd],Di_u),
                                                                 tf.greater_equal(self.Dbi[bd],Di_l)),
@@ -818,18 +884,22 @@ class MPNN(ReaxFF):
                         
                    if (bd in self.belo) or (bdr in self.belo):
                       bd_ = bd if bd in self.belo else bdr
-                      for belo_ in self.belo[bd_]:
-                          r_,Di_l,Di_u,Dj_l,Dj_u,be_l = belo_
-                          fl = tf.where( tf.logical_and(tf.greater_equal(self.rbd[bd],r_),
+                      np_ = len(self.belo[bd_])-1
+                      for i,belo_ in enumerate(self.belo[bd_]):
+                          if i < np_:
+                             right = self.belo[bd_][i+1][0]
+                             r_,Di_l,Di_u,Dj_l,Dj_u,be_l = belo_
+                             fl = tf.where( tf.logical_and(tf.logical_and(tf.greater_equal(self.rbd[bd],r_),
+                                                        tf.less(self.rbd[bd],right)),
                                              tf.logical_and(
                                                  tf.logical_and(tf.less_equal(self.Dbi[bd],Di_u),
                                                                 tf.greater_equal(self.Dbi[bd],Di_l)),
                                                  tf.logical_and(tf.less_equal(self.Dbj[bd],Dj_u),
                                                                 tf.greater_equal(self.Dbj[bd],Dj_l))
                                                          ) ), 1.0,0.0) ##### 
-                          # fl = tf.where(tf.greater_equal(self.rbd[bd],r_),1.0,0.0) ##### 
-                          pen_e   = tf.reduce_sum(input_tensor=tf.nn.relu((be_l - self.EBD[bd])*fl))
-                          self.penalty_be[bd] = self.penalty_be[bd] + pen_e
+                             # fl = tf.where(tf.greater_equal(self.rbd[bd],r_),1.0,0.0) ##### 
+                             pen_e   = tf.reduce_sum(input_tensor=tf.nn.relu((be_l - self.EBD[bd])*fl))
+                             self.penalty_be[bd] = self.penalty_be[bd] + pen_e
 
                 penalty = tf.add(self.penalty_be[bd]*self.lambda_bd,penalty) 
 
@@ -849,9 +919,9 @@ class MPNN(ReaxFF):
                    bo_l = self.spv_bo[bd_][1]
                    bo_u = self.spv_bo[bd_][2]
                    fe   = tf.where(tf.less_equal(self.rbd[bd],r),1.0,0.0) ##### r< r_e that bo > bore_
-                   self.penalty_bo[bd] += tf.reduce_sum(input_tensor=tf.nn.relu((bo_l-self.bo0[bd])*fe))
+                   self.penalty_bo[bd] += tf.reduce_sum(input_tensor=tf.nn.relu((bo_l-self.bop[bd])*fe))
                    fe   = tf.where(tf.greater_equal(self.rbd[bd],r),1.0,0.0) ##### r< r_e that bo > bore_
-                   self.penalty_bo[bd] += tf.reduce_sum(input_tensor=tf.nn.relu((self.bo0[bd]-bo_u)*fe))
+                   self.penalty_bo[bd] += tf.reduce_sum(input_tensor=tf.nn.relu((self.bop[bd]-bo_u)*fe))
 
                 penalty  = tf.add(self.penalty_bo[bd]*self.lambda_bd,penalty) 
 
@@ -1190,14 +1260,14 @@ def set_matrix(m_,spec,bonds,mfopt,mpopt,bdopt,messages,
     reuse_m = True if (bo_layer==bo_layer_ and BOFunction_==BOFunction) else False
 
     if BOFunction!=0:
-        for p_ in ['fsi','fpi','fpp']:
-            if bo_universal_nn is not None:
-               set_universal_wb(m_=m_,pref=p_,bd=bo_universal_nn[0],reuse_m=reuse_m,
-                                nin=1,nout=1,layer=bo_layer,
-                                nnopt=mpopt[0],bias=-1.0)
+       for p_ in ['fsi','fpi','fpp']:
+           if bo_universal_nn is not None:
+              set_universal_wb(m_=m_,pref=p_,bd=bo_universal_nn[0],reuse_m=reuse_m,
+                               nin=1,nout=1,layer=bo_layer,
+                               nnopt=mpopt[0],bias=-1.0)
 
-            set_wb(m_=m_,pref=p_,reuse_m=reuse_m,nin=1,nout=1,layer=bo_layer,
-                   vlist=bonds,nnopt=mpopt[0],bias=-1.0)
+           set_wb(m_=m_,pref=p_,reuse_m=reuse_m,nin=1,nout=1,layer=bo_layer,
+                  vlist=bonds,nnopt=mpopt[0],bias=-1.0)
 
     ############ set weight and bias for message neural network ###################
     if (mf_layer==mf_layer_ and  EnergyFunction==EnergyFunction_
@@ -1296,8 +1366,8 @@ def get_universal_nn(spec,bonds,bo_universal_nn,be_universal_nn,vdw_universal_nn
        else:
           universal_bonds = mf_universal_nn
        for sp in universal_bonds:
-           for t in range(1,messages+1):
-               universal_nn.append('fm'+'_'+sp) # +str(t)
+           # for t in range(1,messages+1):
+           universal_nn.append('fm'+'_'+sp) # +str(t)
     return universal_nn
 
                  
\ No newline at end of file
