@@ -1,10 +1,9 @@
 import numpy as np
 from os import system, getcwd, chdir,listdir
-from os.path import isfile
-
-from pyrsistent import v
-from .dft.cpmd import get_lattice
-from .dft.nwchem import get_nw_gradient,out_xyz
+#from os.path import isfile
+#from pyrsistent import v
+#from .dft.cpmd import get_lattice
+#from .dft.nwchem import get_nw_gradient,out_xyz
 from ase import Atoms
 from ase.io.trajectory import TrajectoryWriter,Trajectory
 from .md.gulp import write_gulp_in,get_reaxff_q
@@ -38,12 +37,13 @@ def taper(r,rmin=0.001,rmax=0.002):
 def get_data(structure='data',direc=None,
              vdwcut=10.0,rcut=None,rcuta=None,
              hbshort=6.5,hblong=7.5,
-             dft='nwchem',atoms=None,
+             atoms=None,
              batch=1000,sample='random',
              minib=100,variable_batch=False,
              p=None,spec=None,bonds=None,angs=None,
              tors=None,hbs=None,
-             sort=False,nindex=[]):
+             #screen=False,
+             nindex=[]):
     t = direc.split('.')
     if t[-1]=='traj':
        dft='ase'
@@ -52,11 +52,12 @@ def get_data(structure='data',direc=None,
                      vdwcut=vdwcut,
                      rcut=rcut,rcuta=rcuta,
                      hbshort=hbshort,hblong=hblong,
-                     dft=dft,atoms=atoms,
-                     batch=batch,minib=minib,sort=sort,
+                     atoms=atoms,
+                     batch=batch,minib=minib,
                      variable_batch=variable_batch,
                      p=p,spec=spec,bonds=bonds,angs=angs,
                      tors=tors,hbs=hbs,
+                     #screen=screen,
                      nindex=nindex)
     return data
 
@@ -87,22 +88,14 @@ class reax_data(object):
   def __init__(self,structure='cl20mol',botol=0.001,direc=None,
                vdwcut=10.0,rcut=None,rcuta=None,
                hbshort=6.75,hblong=7.5,
-               dft='ase',atoms=None,
+               atoms=None,
                batch=1000,minib=100,sample='uniform',
                p=None,spec=None,bonds=None,angs=None,
                tors=None,hbs=None,
-               sort=False,
-               traj=False,
+               traj=False,          
                variable_batch=False,
                nindex=[]):
-      self.sort  = sort
       self.structure = structure
-
-      if direc.find('.traj')>=0:
-         self.dft= 'ase'
-      else:
-         self.dft= dft
-
       self.energy_nw,self.energy_bop = [],[]
       self.table     = []
       self.atom_name = []
@@ -121,35 +114,19 @@ class reax_data(object):
       self.r_cut     = rcut
       self.rcuta     = rcuta
       self.traj      = traj
-      self.cell      = np.array([(10, 0, 0), (0, 10, 0), (0, 0, 10)])
+      # self.screen  = screen             #   screen zero three-body and four-body interaction
       self.status    = True
 
       print('-  Getting informations from directory {:s} ...\n'.format(direc))
-
       cdir = getcwd()
-      if direc is None:
-         outs = listdir(cdir)
-      else:
-         if self.dft!='ase':
-            chdir(direc)
 
-      if self.dft=='nwchem':
-         outs = listdir(direc)
-         xs = self.get_nw_data(outs)
-      elif self.dft=='cpmd':
-         self.get_cpmd_frame()
-      elif self.dft=='siesta':
-         self.get_siesta_energy()
-      elif self.dft=='ase':
-         if atoms==None:
-            images  = self.get_ase_energy(direc)
-            trajonly= False
-         else:
-            images      = [atoms]
-            self.nframe = 1
-            trajonly    = True
+      if atoms==None:
+         images  = self.get_ase_energy(direc)
+         trajonly= False
       else:
-         print('-  Not supported yet!')
+         images      = [atoms]
+         self.nframe = 1
+         trajonly    = True
       
       if variable_batch:
          if self.nframe>batch:
@@ -179,61 +156,37 @@ class reax_data(object):
             indices   = indexs.argsort()
             self.indexs = indexs[indices]
 
-         if self.dft=='cpmd':
-            xs = self.get_cpmd_data()
-         elif self.dft=='siesta':
-            xs = self.get_siesta_cart()
-            qs = self.get_siesta_charges()
-         elif self.dft=='ase':
-            self.get_ase_data(images,trajonly)
-
-         if self.dft!='ase':
-            xs         = np.array(xs)
-            self.x     = xs[self.indexs]
-            self.q_dft = qs[self.indexs]
-
-            energy_nw  = np.array(self.energy_nw)
-            self.energy_nw = energy_nw[self.indexs]
-            self.nframe=self.batch
+         self.get_ase_data(images,trajonly)
       else:
-         if self.dft=='ase':
-            print('-  data set of {:s} is not sufficient, repeat frames ...'.format(self.structure))
-            self.nframe = len(pool)
-            nb = int(self.batch/self.nframe+1)
-            for i in range(nb):
-                # print('-  in direc: %s' %self.structure)
-                if self.nframe*(i+1)<self.batch:
-                   if i==0:
-                      self.indexs = pool
-                   else:
-                      self.indexs = np.append(self.indexs,pool)
-                else:
-                   left = self.batch - self.nframe*i   
-                   if left>0:
-                      self.indexs = np.append(self.indexs,pool[:left])
-
-            self.get_ase_data(images,trajonly)
-            self.nframe=self.batch
-         else:
-            print('-  data set of {:s} is not sufficient, discarded ... ... ...'.format(self.structure))
-            self.status = False
-            if not direc is None:
-               chdir(cdir)
-            return None
+         print('-  data set of {:s} is not sufficient, repeat frames ...'.format(self.structure))
+         self.nframe = len(pool)
+         nb = int(self.batch/self.nframe+1)
+         for i in range(nb):
+               # print('-  in direc: %s' %self.structure)
+               if self.nframe*(i+1)<self.batch:
+                  if i==0:
+                     self.indexs = pool
+                  else:
+                     self.indexs = np.append(self.indexs,pool)
+               else:
+                  left = self.batch - self.nframe*i   
+                  if left>0:
+                     self.indexs = np.append(self.indexs,pool[:left])
 
+         self.get_ase_data(images,trajonly)
+         self.nframe=self.batch
+  
       self.set_parameters()
       self.min_e     = min(self.energy_nw)
       self.max_e     = max(self.energy_nw)
 
       # self.natom_images = self.natom                                    # all atoms include images
       r_  = max(self.r_cut.values())* 2.0
-      R_  = np.sqrt(np.sum(self.cell*self.cell,axis=1))
-
+      R_  = np.max(np.sqrt(np.sum(self.cell*self.cell,axis=2)))    
       # self.image_mask = [0,0,0]
-      for i in range(3):
-          if R_[i]<r_:
-             #self.image_mask[i] = 1
-             print('*  Warning: cell length is to small, using images!')
+      if R_<r_:
+         #self.image_mask[i] = 1
+         raise RuntimeError('-  Error: cell length must lager than 2.0*r_cut, using supercell!')
              
       self.R,self.vr = self.compute_bond(self.x)
       image_rs = self.compute_image(self.vr)                            # vdw interaction images
@@ -247,10 +200,6 @@ class reax_data(object):
       self.compute_hbond(image_rs)
       self.compute_vdw(image_rs)
 
-      if not direc is None:
-         if self.dft!='ase':
-            chdir(cdir)
-
       # self.get_gulp_energy()
       self.get_charge()
       self.get_ecoul(image_rs)
@@ -269,7 +218,7 @@ class reax_data(object):
       hfcell    = 0.5 
       u         = np.linalg.inv(self.cell)
       x         = np.array(x)                       #  project to the fractional coordinate
-      xf        = np.dot(x,u) 
+      xf        = np.matmul(x,u) 
 
       xj   = np.expand_dims(xf,axis=1)
       xi   = np.expand_dims(xf,axis=2)
@@ -285,7 +234,7 @@ class reax_data(object):
          lm = np.where(vr-hfcell>0)
          lp = np.where(vr+hfcell<0)
       
-      vr  = np.dot(vr,self.cell) # convert to ordinary coordinate
+      vr  = np.matmul(vr,np.expand_dims(self.cell,axis=1)) # convert to ordinary coordinate
       R   = np.sqrt(np.sum(vr*vr,axis=3),dtype=np.float32)
       return R,vr 
 
@@ -329,7 +278,7 @@ class reax_data(object):
              print('---    number of {:4d} atom in molecule {:18s} ---'.format(self.natom,self.structure))
              print('-----------------------------------------------------------------\n')
           else:
-             print('-  compute table of batch {0}/{1} ...\r'.format(nf,self.batch),end='\r')
+             print('-  compute table of batch {0}/{1} ...\r'.format(nf+1,self.batch),end='\r')
              for na,tab in enumerate(table):
                  for atom in tab:
                      if not atom in self.table[na]:
@@ -503,7 +452,8 @@ class reax_data(object):
                       elif t6 in self.tors:
                          t = t6    
                       else:
-                         raise RuntimeError('-  torsion angle {:s} not in list!'.format(t1))
+                         # raise RuntimeError('-  torsion angle {:s} not in list!'.format(t1))
+                         continue
                       
                       if t not in torsion:
                          torsion[t] = []
@@ -658,11 +608,12 @@ class reax_data(object):
              self.cos2w = np.concatenate((self.cos2w,c2wtm),axis=1) 
 
   def compute_image(self,vr):
-      vr_ = []
+      vr_   = []
+      cell_ = np.expand_dims(np.expand_dims(self.cell,axis=1),axis=1)
       for i in range(-1,2):
           for j in range(-1,2):
               for k in range(-1,2):
-                  cell = self.cell[0]*i + self.cell[1]*j+self.cell[2]*k
+                  cell = cell_[:,:,:,0]*i + cell_[:,:,:,1]*j+cell_[:,:,:,2]*k
                   vr_.append(vr+cell)
       return vr_
 
@@ -697,7 +648,7 @@ class reax_data(object):
           else:
              vr_ = vr[:,vi_p,vj_p,:]
           r_  = np.sqrt(np.sum(np.square(vr_),axis=2),dtype=np.float32)
-          
+
           ind = np.where(np.logical_and(np.min(r_,axis=0)<=self.vdwcut, 
                                         np.max(r_,axis=0)>0.000001))
           ind = np.reshape(ind,[-1])
@@ -713,7 +664,7 @@ class reax_data(object):
              self.rv = rv_
           else:
              self.rv = np.append(self.rv,rv_,axis=1)
-      
+
       self.rv = np.transpose(self.rv,[1,0])
       self.nvb= len(self.rv)
 
@@ -878,12 +829,14 @@ class reax_data(object):
   def get_gulp_energy(self):
       q,ecoul,eself,evdw = [],[],[],[]
       print('-  get charges from gulp ... \n')
+      A = Atoms(symbols=self.atom_name,
+                positions=self.x[0],
+                cell=self.cell[0],
+                pbc=(1, 1, 1))
       for nf in range(self.batch):
           #print('*  get charges of batch {0}/{1} ...\r'.format(nf,self.batch),end='\r')
-          A = Atoms(symbols=self.atom_name,
-                    positions=self.x[nf],
-                    cell=self.cell,
-                    pbc=(1, 1, 1))
+          A.set_positions(self.x[nf])
+          A.set_cell(self.cell[nf])
           write_gulp_in(A,runword='gradient nosymmetry conv qite verb')
           system('gulp<inp-gulp>out')
           q_,ec_,es_,ev_=get_reaxff_q(self.natom,fo='out')
@@ -901,18 +854,18 @@ class reax_data(object):
       print('-  get charges by QEq ... \n')
       A = Atoms(symbols=self.atom_name,
           positions=self.x[0],
-          cell=self.cell,
+          cell=self.cell[0],
           pbc=(1, 1, 1))
       Qe= qeq(p=self.p,atoms=A)
 
       for nf in range(self.batch):
           # print('*  get charges of batch {0}/{1} ...\r'.format(nf,self.batch),end='\r')
-          A = Atoms(symbols=self.atom_name,
-                    positions=self.x[nf],
-                    cell=self.cell,
-                    pbc=(1, 1, 1))
-          positions = A.get_positions()
-          cell      = A.get_cell()
+          #  A = Atoms(symbols=self.atom_name,
+          #            positions=self.x[nf],
+          #            cell=self.cell[nf],
+          #            pbc=(1, 1, 1))
+          positions = self.x[nf]    # A.get_positions()
+          cell      = self.cell[nf] # A.get_cell()
           Qe.calc(cell,positions)
           q.append(Qe.q[:-1])
       self.q = np.array(q)
@@ -956,49 +909,6 @@ class reax_data(object):
            np.divide(20.0,vdwcut**7.0)*(r**7.0)
       return tp
 
-  def sort_data(self):
-      ''' for cpmd data '''
-      system('mv ENERGIES ENERGIES.orig')
-      system('mv TRAJECTORY TRAJECTORY.orig')
-
-      ft = open('TRAJECTORY.orig','r')
-      fto = open('TRAJECTORY','w')
-      first = None
-      for il,line in enumerate(ft.readlines()):
-          if line.find('<<<<<<  NEW DATA  >>>>>>') >0:
-             fto.close()
-             fto = open('TRAJECTORY','w')
-             first = None
-          else:
-             if first is None:
-                first = line.split()[0]
-             print(line[:-1],file=fto)
-      fto.close()
-      ft.close()
-
-      fe = open('ENERGIES.orig','r')
-      feo = open('ENERGIES','w')
-      for il,line in enumerate(fe.readlines()):
-          if len(line.split())==0:
-             continue
-          if line.split()[0] == first:
-             if il>0:
-                feo.close()
-                feo = open('ENERGIES','w')
-             print(line[:-1],file=feo)
-          else:
-             print(line[:-1],file=feo)
-      feo.close()
-      fe.close()
-
-  def get_cpmd_frame(self):
-      if self.sort:
-         self.sort_data()
-      fe = open('ENERGIES','r')
-      lines = fe.readlines()
-      fe.close()
-      self.nframe = len(lines)
-
   def get_ase_energy(self,direc):
       images = Trajectory(direc)
       self.nframe = len(images)
@@ -1006,13 +916,14 @@ class reax_data(object):
 
   def get_ase_data(self,images,trajonly):
       ''' getting data in the ase traj '''
-      self.x         = []
-      self.forces    = []
-      self.energy_nw = []
+      x         = []
+      cell      = []
+      forces    = []
+      energy_nw = []
       for i,ind_ in enumerate(self.indexs):
           imag = images[ind_]
           if i==0:
-             self.cell      =imag.get_cell()
+             # self.cell    = imag.get_cell()
              self.atom_name = imag.get_chemical_symbols()
              self.natom     = len(self.atom_name)
 
@@ -1020,292 +931,20 @@ class reax_data(object):
              e = 0.0
           else:
              e = imag.get_potential_energy()
-
-          self.x.append(imag.positions)
+          cell.append(imag.get_cell())
+          x.append(imag.positions)
           try:
              force_ = imag.get_forces()
           except:
              force_ = None
              # print('-  ignoring the forces as are not available.')
-          self.forces.append(force_)
-          self.energy_nw.append(e)
-      self.energy_nw = np.array(self.energy_nw)
-      self.x = np.array(self.x)
-      self.forces = np.array(self.forces)
-
-  def get_siesta_energy(self,label='siesta'):
-      fe = open(label+'.MDE','r')
-      lines = fe.readlines()
-      fe.close()
-      l1= lines[1].split()
-      l = lines[-1].split()
-      if len(l)==0:
-         l = lines[-2].split()
-      self.nframe = int(l[0]) - int(l1[0]) + 1
-
-      self.energy_nw = [] # np.zeros([self.nframe],dtype=np.float32)
-      for line in lines:
-          l = line.split()
-          if l[1] != 'Step':
-             # f = int(l[0]) - 1
-             self.energy_nw.append(float(l[2])) # unit eV
-
-  def get_siesta_cart(self,label='siesta'):
-      fin = open('in.fdf','r') 
-      lines= fin.readlines()
-      fin.close()           # getting informations from input file
-
-      for i,line in enumerate(lines):
-          l = line.split()
-          if len(l)>0:
-             if l[0] == 'NumberOfSpecies':
-                ns = int(l[1])
-             if l[0] == 'NumberOfAtoms':
-                self.natom = int(l[1])
-             if l[0]=='%block':
-                if l[1]=='ChemicalSpeciesLabel':
-                   spl = i+1
-                if l[1]=='AtomicCoordinatesAndAtomicSpecies':
-                   atml= i+1
-
-      sp = []
-      for isp in range(ns):
-          l = lines[spl+isp].split() 
-          sp.append(l[2])
-
-      for na in range(self.natom):
-           l = lines[atml+na].split() 
-           self.atom_name.append(sp[int(l[3])-1])
-
-      fe = open(label+'.MD_CAR','r')
-      lines = fe.readlines()
-      fe.close()
-      nl = len(lines)
-      if nl-(self.natom+7)*self.nframe!=0:
-         fra = (nl-(self.natom+7)*self.nframe)/(self.natom+7)
-         raise RuntimeError('-  %d frames more than expected, error case ... ... ...' %fra)
-
-
-      lsp = lines[5].split()
-      nsp = [int(l) for l in lsp]
-      xs = []
-      if self.traj:
-         his = TrajectoryWriter(self.structure+'.traj',mode='w')
-      for nf in range(self.nframe):
-          block = self.natom + 7
-          nl = block*nf
-          la = lines[nl+2].split()
-          lb = lines[nl+3].split()
-          lc = lines[nl+4].split()
-
-          a = [float(la[0]),float(la[1]),float(la[2])]
-          b = [float(lb[0]),float(lb[1]),float(lb[2])]
-          c = [float(lc[0]),float(lc[1]),float(lc[2])]
-          x = []
-          il= 0
-          for i,s in enumerate(nsp):
-              for ns in range(s):
-                  l = lines[nl+7+il].split()
-                  xd = [float(l[0]),float(l[1]),float(l[2])]
-
-                  x1 = xd[0]*a[0]+xd[1]*b[0]+xd[2]*c[0]
-                  x2 = xd[0]*a[1]+xd[1]*b[1]+xd[2]*c[1]
-                  x3 = xd[0]*a[2]+xd[1]*b[2]+xd[2]*c[2]
-
-                  x.append([x1,x2,x3])
-                  il += 1
-          xs.append(x)
-          if self.traj:
-             A = Atoms(self.atom_name,x,cell=[a,b,c],pbc=[True,True,True])
-             his.write(atoms=A)
-
-      self.cell=np.array([a,b,c])
-
-      if self.traj: 
-         his.close()
-      return xs
-
-  def get_siesta_charges(self,label='siesta'):
-      fo = open(label+'.out','r') 
-      lines= fo.readlines()
-      fo.close()            # getting informations from .out file
-      qs  = []
-
-      spec_atoms = {}
-      obs        = {}
-      for s in self.spec:
-          spec_atoms[s] = []
-
-
-      for i,s in enumerate(self.atom_name):
-          spec_atoms[s].append(i) 
-
-      nsp = {}
-      for s in spec_atoms:
-          nsp[s] = len(spec_atoms[s])
-
-      iframe = 0
-      spec_  = []
-      for i,line in enumerate(lines):
-          if line.find('Begin MD step')>=0:
-             frame = int(line.split()[4])-1
-         
-          if line.find('mulliken: Atomic and Orbital Populations:')>=0:
-             # print('-  current frame %d, MD step %d...' %(iframe,frame))
-             if iframe==0:
-                cl    = 0
-                end_  = True
-                while end_:
-                      cl   += 1
-                      line_ = lines[i+cl]
-                      if line_.find('mulliken: Qtot')>=0:
-                         end_ = False
-
-                      if line_.find('Species:')>=0:
-                         sl = 0
-                         spec_.append(line_.split()[1])
-                         
-                         qline_ = lines[i+cl+1]
-                         if qline_.find('Atom  Qatom  Qorb')<0:
-                            print('-  an error case ... ... ')
-
-                         qline_ = lines[i+cl+2]
-                         ql_    = qline_.split()
-                         nob    = len(ql_)
-
-                         o      = 0
-                         spec_end = True
-                         while spec_end:
-                               o += 1
-                               qline_ = lines[i+cl+2+o]
-                               ql_    = qline_.split()
-                               if len(ql_)== nob+2:
-                                  obs[spec_[-1]] = o
-                                  spec_end = False
-
-                # print('\n Qorb: \n',obs)
-                q_     = np.zeros([self.natom])
-                cl    = 0
-                end_  = True
-                while end_:
-                      cl   += 1
-                      line_ = lines[i+cl]
-                      if line_.find('mulliken: Qtot')>=0:
-                         end_ = False
-
-                      if line_.find('Species:')>=0:
-                         sl = 0
-                         s_ = line_.split()[1]
-                         # print('\n-  charges of species: %s \n' %s_)
-                         
-                         for i_ in range(nsp[s_]):
-                             qline_ = lines[i+cl+2+(i_+1)*obs[s_]]
-                             ql_    = qline_.split()
-
-                             ai     = int(ql_[0])-1
-                             q_[ai] = float(ql_[1])-self.p['vale_'+s_]
-
-                         cl += 2+i_*obs[s_]
-                qs.append(q_)
-             else:
-                q_    = np.zeros([self.natom])
-                cl    = 0
-                end_  = True
-                while end_:
-                      cl   += 1
-                      line_ = lines[i+cl]
-                      if line_.find('mulliken: Qtot')>=0:
-                         end_ = False
-
-                      if line_.find('Species:')>=0:
-                         sl = 0
-                         s_ = line_.split()[1]
-
-                         for i_ in range(nsp[s_]):
-                             qline_ = lines[i+cl+2+(i_+1)*obs[s_]]
-                             ql_    = qline_.split()
-                             
-                             ai     = int(ql_[0])-1
-                             q_[ai] = float(ql_[1])-self.p['vale_'+s_]
-
-                         cl += 2+i_*obs[s_]
-                qs.append(q_)
-
-             iframe += 1
-      return np.array(qs)
-
-  def get_cpmd_data(self):
-      fe = open('ENERGIES','r')
-      lines = fe.readlines()
-      fe.close()
-      nl = len(lines)
-      xs = []
-      self.energy_nw = []
-      if isfile('inp-nve'):
-         inp='inp-nve'
-      elif isfile('inp-nvt'):
-         inp='inp-nvt'
-      else: 
-         raise FileNotFoundError('-  error: inp-nvt or inp-nve file not found!')
-
-      cell = get_lattice(inp=inp)  # surpose the cell is orthogonal
-      self.cell = np.array(cell)
-      
-      for nf in range(nl):
-          l = lines[nf].split()
-          self.energy_nw.append(float(l[3])* 27.211396) ### a.u. to eV
-
-      fg = open('GEOMETRY.xyz','r')
-      lines = fg.readlines()
-      fg.close()
-      self.natom = len(lines) - 2
-      for na in range(2,self.natom+2):
-          self.atom_name.append(lines[na].split()[0])
-
-      ft = open('TRAJECTORY','r')
-      lines = ft.readlines()
-      ft.close()
-
-      if self.traj:
-         his = TrajectoryWriter(self.structure+'.traj',mode='w')
-
-      for nf in range(nl):
-          x = []
-           
-          for na in range(self.natom):
-              l = lines[nf*self.natom+na].split()
-              x.append([float(l[1])*0.52917721067,float(l[2])*0.52917721067,float(l[3])*0.52917721067])
-          xs.append(x)
-
-          if self.traj:
-             if nf in self.indexs:
-                A = Atoms(self.atom_name,x,cell=self.cell,pbc=[True,True,True])
-                his.write(atoms=A)
-
-      if self.traj: 
-         his.close()
-      return xs
-
-  def get_nw_data(self,outs):
-      i,ii = 0,0
-      xs = []
-      for out in outs:
-          if out.find('.out')>=0:
-             print('-  Getting molecular informations and nwchem datas from %s' %out)
-             natm,atoms,X = out_xyz(out_file=out)
-
-             xyz = out[:-4] + '.xyz'
-             e, gradient = get_nw_gradient(out)
-
-             if e is None or gradient is None:
-                continue
-             xs.append(X)
-             self.energy_nw.append(e)
-             ii += 1
-      self.nframe = ii
-      self.natom = natm
-      self.atom_name = atoms
-      return xs
+          forces.append(force_)
+          energy_nw.append(e)
+      self.energy_nw = np.array(energy_nw)
+      self.x         = np.array(x)
+      self.cell      = np.array(cell)
+      self.forces    = np.array(forces) # ,dtype=np.float32)
+      # print(len(self.x),len(self.cell),len(self.forces))
 
   def set_parameters(self):
       self.P={}
