@@ -6,6 +6,7 @@ import argh
 import argparse
 import numpy as np
 from .reax import ReaxFF
+from .reax_nn import ReaxFF_nn
 from .mpnn import MPNN
 from .initCheck import Init_Check
 from .dingtalk import send_msg
@@ -30,10 +31,10 @@ def train_reax(dataset=None,step=5000,batch=None,convergence=0.97,lossConvergenc
           vdw_layer=None,
           VdwFunction=0,
           BOFunction=0,
-          be_univeral_nn='all',
-          bo_univeral_nn='all',
-          mf_univeral_nn='all',
-          vdw_univeral_nn='all',
+          be_universal_nn='all',
+          bo_universal_nn='all',
+          mf_universal_nn='all',
+          vdw_universal_nn='all',
           spv_bo=False,#boc={},#boup={},
           spv_be=False,belo={},beup={},
           spv_vdw=False,vlo={},vup={},
@@ -43,7 +44,7 @@ def train_reax(dataset=None,step=5000,batch=None,convergence=0.97,lossConvergenc
           weight={'others':10.0},
           lambda_bd=10000.0,lambda_reg=0.0001,lambda_ang=0.02,
           learning_rate=1.0e-4,
-          ffield='ffield.json'):
+          ffield='ffield.json',**kwargs):
     ''' training the force field '''
     rn = ReaxFF(libfile=ffield,
                 dataset=dataset, 
@@ -52,7 +53,6 @@ def train_reax(dataset=None,step=5000,batch=None,convergence=0.97,lossConvergenc
                 optword=optword,
                 opt=None,clip=clip,
                 nn=nn,
-                pkl=False,
                 batch_size=batch,
                 losFunc='n2',
                 lambda_bd=lambda_bd,
@@ -90,8 +90,8 @@ def train_mpnn(dataset=None,step=5000,batch=None,convergence=0.97,lossConvergenc
                spec=[],
                writelib=1000,nn=True,vdwnn=True,VdwFunction=1,
                bo_layer=[4,1],mf_layer=[9,2],be_layer=[6,1],vdw_layer=[6,1],
-               be_univeral_nn='all',bo_univeral_nn='all',mf_univeral_nn='all',
-               vdw_univeral_nn='all',
+               be_universal_nn='all',bo_universal_nn='all',mf_universal_nn='all',
+               vdw_universal_nn='all',
                BOFunction=0,EnergyFunction=3,MessageFunction=2,
                messages=1,mpopt=[1,1,1,1],
                spv_bo=False,#boc={},#boup={},
@@ -102,7 +102,7 @@ def train_mpnn(dataset=None,step=5000,batch=None,convergence=0.97,lossConvergenc
                spv_ang=False,
                lambda_reg=0.0001, # regularize=True,
                learning_rate=1.0e-4,
-               ffield = 'ffield.json'):
+               ffield = 'ffield.json',**kwargs):
     ''' train the massage passing model '''
     regularize =True if lambda_reg>0.0001 else False
     cons_=['val','vale',
@@ -130,15 +130,14 @@ def train_mpnn(dataset=None,step=5000,batch=None,convergence=0.97,lossConvergenc
               nn=nn,vdwnn=vdwnn,VdwFunction=VdwFunction,
               bo_layer=bo_layer,mf_layer=mf_layer,
               be_layer=be_layer,vdw_layer=vdw_layer,
-              be_univeral_nn=be_univeral_nn,
-              bo_univeral_nn=bo_univeral_nn,
-              mf_univeral_nn=mf_univeral_nn,
-              vdw_univeral_nn=vdw_univeral_nn,
+              be_universal_nn=be_universal_nn,
+              bo_universal_nn=bo_universal_nn,
+              mf_universal_nn=mf_universal_nn,
+              vdw_universal_nn=vdw_universal_nn,
               BOFunction=BOFunction,EnergyFunction=EnergyFunction,
               MessageFunction=MessageFunction,
               messages=messages,
               mpopt=mpopt,
-              pkl=False,
               batch_size=batch,
               losFunc='n2',
               regularize_be=regularize,regularize_bo=regularize,regularize_mf=regularize,
@@ -179,6 +178,91 @@ def train_mpnn(dataset=None,step=5000,batch=None,convergence=0.97,lossConvergenc
     rn.close()
     return loss,accu,accMax,p,zpe,i
 
+def train_nn(dataset=None,step=5000,batch=None,convergence=0.97,lossConvergence=1000.0,
+               optword='nocoul',cons=None,clip={},
+               spec=[],
+               writelib=1000,nn=True,vdwnn=True,VdwFunction=1,
+               bo_layer=[4,1],mf_layer=[9,2],be_layer=[6,1],vdw_layer=[6,1],
+               be_universal_nn='all',bo_universal_nn='all',mf_universal_nn='all',
+               vdw_universal_nn='all',
+               BOFunction=0,EnergyFunction=3,MessageFunction=2,
+               messages=1,mpopt=[1,1,1,1],
+               spv_bo=False,#boc={},#boup={},
+               spv_be=False,belo={},beup={},
+               spv_vdw=False,vlo={},vup={},
+               lambda_me=0.1,lambda_ang=0.02,lambda_bd=1000.0,
+               weight={'others':10.0},
+               spv_ang=False,
+               lambda_reg=0.0001, # regularize=True,
+               learning_rate=1.0e-4,
+               ffield = 'ffield.json',**kwargs):
+    ''' train the massage passing model '''
+    regularize =True if lambda_reg>0.0001 else False
+    cons_=['val','vale',
+           'ovun1','ovun2','ovun3','ovun4',
+           'ovun5','ovun6','ovun7','ovun8',
+           'lp2','lp3',#'lp1',
+           'cot1','cot2',
+           'coa1','coa2','coa3','coa4',
+           'pen1','pen2','pen3','pen4',
+           'Depi','Depp',#'Desi','Devdw',
+           #'bo1','bo2','bo3','bo4','bo5','bo6',
+           #'rosi','ropi','ropp',
+           'cutoff','hbtol','acut' ] #'val8','val9','val10' 
+    if cons is None:
+       cons = cons_
+    else:
+       cons = cons + cons_
+       
+    rn = ReaxFF_nn(libfile=ffield,
+              dataset=dataset, 
+              dft='siesta',
+              spec=spec,
+              optword=optword,clip=clip,
+              cons=cons,
+              nn=nn,vdwnn=vdwnn,VdwFunction=VdwFunction,
+              bo_layer=bo_layer,mf_layer=mf_layer,
+              be_layer=be_layer,vdw_layer=vdw_layer,
+              be_universal_nn=be_universal_nn,
+              bo_universal_nn=bo_universal_nn,
+              mf_universal_nn=mf_universal_nn,
+              vdw_universal_nn=vdw_universal_nn,
+              BOFunction=BOFunction,EnergyFunction=EnergyFunction,
+              MessageFunction=MessageFunction,
+              messages=messages,
+              mpopt=mpopt,
+              batch_size=batch,
+              losFunc='n2',
+              regularize_be=regularize,regularize_bo=regularize,regularize_mf=regularize,
+              lambda_reg=lambda_reg,lambda_bd=lambda_bd,
+              spv_be=spv_be,belo=belo,beup=beup,
+              spv_bo=spv_bo,#boup=boup,
+              spv_vdw=spv_vdw,vlo=vlo,vup=vup,
+              spv_ang=spv_ang,lambda_ang=lambda_ang,
+              weight=weight,lambda_me=lambda_me,
+              convergence=convergence,
+              lossConvergence=lossConvergence) # Loss Functon can be n2,abs,mse,huber
+
+    loss,accu,accMax,i,zpe =rn.run(learning_rate=learning_rate,
+                                   step=step,
+                                   print_step=10,
+                                   writelib=writelib) 
+
+    libstep = int(i - i%writelib)
+    if i==libstep:
+       libstep = libstep - writelib
+    if libstep<=0:
+       ffd = 'ffield.json'
+    else:
+       ffd = 'ffield_' + str(libstep) + '.json'
+
+    p   = rn.p_
+    if loss==0.0 and accu==0.0:
+       send_msg('-  Warning: the loss is NaN!')
+       return -1,1,accMax,p,zpe,i
+
+    rn.close()
+    return loss,accu,accMax,p,zpe,i
 
 if __name__ == '__main__':
    ''' use commond like ./bp.py <t> to run it
@@ -186,6 +270,6 @@ if __name__ == '__main__':
        t:   train the whole net
    '''
    parser = argparse.ArgumentParser()
-   argh.add_commands(parser, [train,train_mpnn])
+   argh.add_commands(parser, [train,train_mpnn,train_nn])
    argh.dispatch(parser)
 
